'''
Created on Apr 3, 2020

@author: Reynolds
'''
import unittest
from wrtdk.parser.em.em3d.em.em import emv2


class Test(unittest.TestCase):


    def setUp(self):
        self.msg = b't\xd3V|\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\n'


    def tearDown(self):
        pass


    def testParse(self):
        em = emv2()
        em.set_bins(16)
        
        em.parse(self.msg)
        self.assertFalse(em.hasErrored())


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on Aug 20, 2018

@author: reynolds
'''
import os, sys, datetime, struct, math
from wrtdk.parser.parser import parser

class udp_wrapper(parser):
    '''  a class for parsing the wrt udp 
    sensor message wrapper '''
    
    BYTES = type(b'')

    def __init__(self):
        ''' Constructor '''
        super().__init__()# inherit superclass
        
        # define constants
        self.LENGTH = 36
        self.ID_LENGTH = 6
        self.UTC_TIMESTAMP = datetime.datetime.utcnow().timestamp()
        #print('Current UTC Timestamp',self.UTC_TIMESTAMP)# print timestamp
        
        # intiialzie properties
        self.reset()
        
    def reset(self):
        ''' resets the properties '''
        self._type = -1
        self._status = ''
        self._mcu = -1
        self._length = -1
        self._n = -1
        self._sen = -1
        self._sys = -1
        self._chk = -1
        
    def __str__(self,):
        ''' prints the class as a string '''
        return '%s:[type:%s,sensor:%d,system:%s,status:%s,fiducial:%d,mcu:%f,length:%d,checksum:%d]' % (type(self).__name__,
                                                                                                     self._type,
                                                                                                     self._sen,
                                                                                                     self._sys,
                                                                                                     self._status,
                                                                                                     self._n,
                                                                                                     self._mcu,
                                                                                                     self._length,
                                                                                                     self._chk)
    
    def parse(self,msg):
        ''' parses the message wrapper '''
        self.reset()
        try:
            #m1,m2,m3,m4,m5,m6,_,self._sen,self._sys,self._n,stat1,stat2,mcu1,mcu2,_,self._length,self._chk = struct.unpack('>ssssssHHHHssIIIII',msg)
            #self._type = m1.decode() + m2.decode() + m3.decode() + m4.decode() + m5.decode() + m6.decode()
            #self._status = stat1.decode() + stat2.decode()
            
            self._type,_,self._sen,self._sys,self._n,self._status,mcu1,mcu2,_,self._length,self._chk = struct.unpack('>6s4H2s5I',msg)
            self._type = self._type.decode()
            self._status = self._status.decode()
            
            self._mcu = mcu1 + mcu2/1000.0# s
            #if self._mcu < self.UTC_TIMESTAMP-86400: self._mcu += 86400
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
            
    def get(self,mtype=b'$?????',sen_id=0,sys_id=0,fid=0,status=b'ND',
            t_s=0,t_ms=0,mlen=0,cksm=0):
        try:
            if type(mtype) != self.BYTES: mtype = mtype.encode()
            if type(status) != self.BYTES: status = status.encode()
            return struct.pack('>6s4H2s5I',
                               mtype,0,sen_id,sys_id,fid,
                               status,t_s,t_ms,0,mlen,cksm)
        except Exception as e:
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d.'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno))
            return None
    
    def getStatus(self):
        ''' returns the status '''
        return self._status
    
    def getMCU(self):
        ''' returns the mcu time '''
        return self._mcu
    
    def getTimestamp(self):
        ''' returns the timestamp '''
        return str(datetime.datetime.fromtimestamp(self._mcu))
    
    def getLength(self):
        ''' returns the length '''
        return self._length
    
    def getType(self):
        ''' returns the message type '''
        return self._type
    
    def getMsgStart(self,position=0):
        ''' returns the message start '''
        return self.LENGTH + position
    
    def getMsgEnd(self,position=0):
        ''' returns the message end '''
        return self.LENGTH + self._length + position
    
    def getStart(self,position=0):
        ''' returns the start of the message '''
        return position
    
    def getEnd(self,position=0):
        ''' returns the end of the message '''
        return position + self.LENGTH
    
    def getFiducial(self):
        ''' returns the fiducial '''
        return self._n
    
    def getSensorID(self):
        ''' returns the sensor id '''
        return self._sen
    
    def getSystemID(self):
        ''' returns the system id '''
        return self._sys
    
    def getWrapperChecksum(self):
        ''' returns the wrapper checksum '''
        return self._chk

class wrtsys(parser):
    ''' parses for wrt system message '''

    def __init__(self):
        ''' Constructor '''
        super().__init__()# inherit superclass
        
        # initialize properties
        self.reset()
        
    def reset(self):
        ''' resets the message '''
        self._msg = 'Parse Error'''
        
    def parse(self,msg):
        ''' parses the $SYSTM message '''
        self.reset()
        try:
            tmp = msg.decode()
            self._msg = tmp[0:3] + ' ' + tmp[5:7] + '/' + tmp[3:5] + '/' + tmp[7:9] + ' ' + tmp[9:11] + ':' + tmp[11:13] + ':' + tmp[13:18] + ' ' +tmp[18:]
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
            
    def getData(self):
        ''' returns the data '''
        return [self._msg]
    
class rtcom_mag(parser):
    ''' message parser for a real time compensated mag '''
    
    def __init__(self):
        ''' constructor '''
        super().__init__()
        
        # reset the parser
        self.reset()
        
    def reset(self):
        ''' resets the value '''
        self._com = self._nan()
        
    def parse(self,msg):
        ''' parses a real time compensated mag message '''
        self.reset()
        
        try:
            self._com, = struct.unpack('<d',msg)
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
            
    def getData(self):
        ''' returns the data 
        0) compensated mag data nT'''
        return [self._com]
        
class rtdet_mag(parser):
    ''' message parser for a real time compensated mag '''
    
    def __init__(self):
        ''' constructor '''
        super().__init__()
        
        # reset the parser
        self.reset()
        
    def reset(self):
        ''' resets the value '''
        self._flag = self._nan()
        self._strength = self._nan()
        
    def parse(self,msg):
        ''' parses a real time compensated mag message '''
        self.reset()
        
        try:
            self._flag,self._lat,self._lon,self._depth,self._alt,self._strength = struct.unpack('<dddddd',msg)
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
            
    def getData(self):
        ''' returns the data 
        0) detection flag
        1) Vehicle Latitude (dd)
        2) Vehicle Longitude (dd)
        3) Vehicle Depth (m)
        4) Vehicle Altitude above seafloor (m)
        5) detection strength (nT)'''
        return [self._flag,self._lat,self._lon,self._depth,self._alt,self._strength]
        
class rtinv_mag(parser):
    ''' message parser for a real time compensated mag '''
    
    def __init__(self):
        ''' constructor '''
        super().__init__()
        
        # reset the parser
        self.reset()
        
    def reset(self):
        ''' resets the value '''
        self._x = self._nan()
        self._y = self._nan()
        self._z = self._nan()
        self._s = self._nan()
        self._h = self._nan()
        self._mx = self._nan()
        self._my = self._nan()
        self._mz = self._nan()
        self._m = self._nan()
        self._fit = self._nan()
        self._navE = self._nan()
        self._navN = self._nan()
        self._navA = self._nan()
        
    def parse(self,msg):
        ''' parses a real time compensated mag message '''
        self.reset()
        
        try:
            self._x,self._y,self._z,self._s,self._h,self._mx,self._my,self._mz,self._fit,self._navE,self._navN,self._navA = struct.unpack('<dddddddddddd',msg)
            self._m = math.sqrt(self._mx ** 2 + 
                                self._my ** 2 +
                                self._my ** 2)
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
            
    def getData(self):
        ''' returns the data 
        0) inversion x location m
        1) inversion y location m
        2) inversion z location m
        3) inversion speed in mps
        4) inversion headig in degrees
        5) moment x in A m^2
        6) moment y in A m^2
        7) moment z in A m^2
        8) data model fit
        9) navigation easting m
        10) navigation northing m
        11) navigation altitude m
        12) moment magnitude in A m^2'''
        return [self._x,self._y,self._z,
                self._s,self._h,
                self._mx,self._my,self._mz,
                self._fit,
                self._navE,self._navN,self._navA,
                self._m]
    
class rttrk_mag(parser):
    ''' message parser for a real time compensated mag '''
    
    LENGTH = 5
    
    def __init__(self):
        ''' constructor '''
        super().__init__()
        
        # reset the parser
        self.reset()
        
    def reset(self):
        ''' resets the value '''
        self._x = self._nan()
        self._y = self._nan()
        self._z = self._nan()
        self._m = self._nan()
        self._n = self._nan()
        
    def parse(self,msg):
        ''' parses a real time compensated mag message '''
        self.reset()
        
        try:
            self._x,self._y,self._z,self._m,self._n = struct.unpack('<ddddd',msg)
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
            
    def getData(self):
        ''' returns the data '''
        return [self._x,self._y,self._z,self._m,self._n]
    
class sw_mag(rttrk_mag):
    
    def parse(self,msg):
        ''' parses a real time compensated mag message '''
        self.reset()
        
        try:
            self._x,self._y,self._z,self._m,self._n = struct.unpack('>ddddd',msg)
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
    
def test_systm():
    qmag = wrtsys()
    filename = r'C:\Users\reynolds\Documents\1606_navsea\data\no_i2c.dat'
    msg = get_msg(filename,b'$SYSTM')
    print('msg:%s' % msg)
    qmag.parse(msg)
    print('$SYSTM',qmag.getData())
    
def test_rtcom_mag():
    qmag = rtcom_mag()
    filename = r'C:\Users\reynolds\Documents\1704_madunit\data\20190220_realtimeinv\RT_MSG.bin'
    msg = get_msg(filename,b'$RTCOM')
    print('msg:%s' % msg)
    qmag.parse(msg)
    print('$RTCOM',qmag.getData())
    
def test_rtdet_mag():
    qmag = rtdet_mag()
    filename = r'C:\Users\reynolds\Documents\1704_madunit\data\20190220_realtimeinv\RT_MSG.bin'
    msg = get_msg(filename,b'$RTDET')
    print('msg:%s' % msg)
    qmag.parse(msg)
    print('$RTDET',qmag.getData())
    
def test_rtinv_mag():
    qmag = rtinv_mag()
    filename = r'C:\Users\reynolds\Documents\1704_madunit\data\20190220_realtimeinv\RT_MSG.bin'
    msg = get_msg(filename,b'$RTINV')
    print('msg:%s' % msg)
    qmag.parse(msg)
    print('$RTINV',qmag.getData())
    
def test_rtrtk_mag():
    qmag = rttrk_mag()
    filename = r'C:\Users\reynolds\Documents\1704_madunit\data\20190226_rtmsgs\RT_MSG.bin'
    msg = get_msg(filename,b'$RTTRK')
    qmag.parse(msg)
    print('$RTTRK',qmag.getData())
    
def get_msg(filename,msgid):
    with open(filename,'rb') as f:
        idlen = 6
        msg = f.read(idlen)
        pos = 6
        
        while msg != msgid:
            msg = msg[1::] + f.read(1)
            pos = pos + 1
            
        wrapper = udp_wrapper()
            
        msg = msg + f.read(36-6)
        pos = pos + 36 - 6
        wrapper.parse(msg)
        
        print(msg)
        print(type(wrapper.getSensorID()),type(wrapper.getSystemID()))
        
        payload = f.read(wrapper.getLength())
        pos = pos + 1
        
        return payload
            
if __name__ == '__main__':
    test_systm()
    test_rtcom_mag()
    test_rtdet_mag()
    test_rtinv_mag()
    test_rtrtk_mag()


'''
Created on Feb 20, 2019

@author: reynolds
'''

import os, sys

class simulator(object):
    ''' A data simulator class for simulating sensors'''

    def __init__(self):
        ''' Constructor '''
        super().__init__()
        self.reset()
        
    def reset(self):
        ''' resets the simulator '''
        self._len = 0
        self._i = -1
        self._error = False
        self._msgs = []
        self._dt = []
            
    def read(self,filename):
        ''' reads the file and parses the messages '''
        self.reset()
        print('Reading %s ... ' % filename)
        if not os.path.exists(filename):
            self._error = True
            print('Error. File does not exist.')
            return False
        
    def current(self):
        ''' returns the current message number '''
        return self._i
    
    def length(self):
        ''' returns the total number of messages '''
        return self._len
        
    def getNext(self):
        ''' gets the next message '''
        self._i += 1
        if self._i == self._len: self._i = 0
        try:
            return (self._msgs[self._i],self._dt[self._i])
        except Exception as e:
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno))
            self._i = 0
            return (self._msgs[self._i],self._dt[self._i])
        
    def hasNext(self):
        ''' returns whether there is another message '''
        return self._i < self._len
        
    def didError(self):
        ''' returns whether an error did occur '''
        return self._error


'''
Created on Dec 17, 2019

@author: Reynolds
'''

import sys

import numpy as np

class geophysical_data(object):
    '''
    classdocs
    '''

    def __init__(self,length=1):
        '''
        length corresponds to the number of systems
        '''
        self.geodata = [np.array([])] * length
        self.vmag = [np.array([])] * length
        self.imu = [np.array([])] * length
        self.lidar = [np.array([])] * length
        self.sonar = [np.array([])] * length
        self.gpgga = [np.array([])] * length
        self.gprmc = [np.array([])] * length
        
if __name__ == '__main__':
    a = geophysical_data()
    d = {'tfmag':np.array([]),
             'vmag':np.array([]),
             'imu':np.array([]),
             'lidar':np.array([]),
             'sonar':np.array([]),
             'gpgga':np.array([]),
             'gprmc':np.array([]) }
    
    print(sys.getsizeof(a),sys.getsizeof(d))
        


'''
Created on Dec 26, 2019

@author: Reynolds
'''
import unittest
from wrtdk.parser.vmag.pni import pni


class Test(unittest.TestCase):


    def setUp(self):
        self.msg = b'\xff\xff\xfb5\xff\xff\xff\x0f\x00\x00\r\xf6:96\x00'
        self.vmag = pni()

    def test_parse(self):
        self.vmag.parse(self.msg)
        self.assertFalse(self.vmag.hasErrored())
        d = self.vmag.getData()
        
        msg = self.vmag.get(bx=d[0], by=d[1], bz=d[2])
        
        self.assertEqual(msg[:-4],self.msg[:-4])


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on Nov 19, 2019

@author: reynolds
'''

import os, sys, struct

import numpy as np

from wrtdk.parser.parser import parser

class conf(parser):
    PREAMBLE = b'CONF'
    
    def __init__(self,version=5,length=27):
        super().__init__()
        self._version = version
        self._mlen = length
        self.reset()
        
    def reset(self):
        self._chksum = self._minus1()
        self._chkver = self._minus1()
        self._conver = self._minus1()
        self._sysid  = self._minus1()
        self._fid    = self._minus1()
        self._svnver = self._minus1()
        self._states = self._minus1()
        self._txcnt  = self._minus1()
        self._systmt = self._minus1()
        self._samplt = self._minus1()
        self._samplc = self._minus1()
        self._thold  = self._minus1()
        self._rxsc   = self._minus1()
        self._axsc   = self._minus1()
        self._bnsc   = self._minus1()
        self._bins   = []
        
    def getData(self):
        '''
        0) checksum
        1) checksum version
        2) conf version
        3) system id
        4) fiducial
        5) svn revision
        6) max tx states
        7) tx on count. NOT THE NUMBER OF TXS
        8) System Timer 
        9) Sample Timer
        10) Timer Holdoff
        11) Rx Cube Count
        12) Axes Count
        13) Bin Count
        14:14+Bin Count) Bins
        '''
        return [self._chksum,
                self._chkver,
                self._conver,
                self._sysid,
                self._fid,
                self._svnver,
                self._states,
                self._txcnt,
                self._systmt,
                self._samplc,
                self._thold,
                self._rxsc,
                self._axsc,
                self._bnsc] + self._bins
                
    def get_fiducial(self):
        return self._fid
        
    def get_bins(self):
        return self._bins
    
    def get_bins_micros(self):
        return  (np.diff( np.array([0] + self._bins) )/2.0 + 
                          np.array([0] + self._bins[:-1]) ) * self.get_sample_time_micros() + self.get_holdoff_micros()
                          
    def get_sample_time_micros(self):
        return float(self._samplt)
    
    def get_holdoff_micros(self):
        return float(self._thold) * float(self._samplt)
    
    def get_length(self):
        return self._mlen + 2 * self._bnsc + 1
    
    def get_version(self):
        return self._version
    
    def get_axes(self):
        return self._axsc
    
    def get_rxs(self):
        return self._rxsc
    
    def get_bin_count(self):
        return self._bnsc
    
    def checksum(self,byte_values):
        return 0
    
    def __str__(self):
        return '%s[error:%b,bins:%d,rxs:%d,axes:%d]' % (type(self).__name__,self._error,self._bns.self._axs)
    
class confv3(conf):
    '''
    classdocs
    '''
    
    _LENGTH = 24
    _VERSION = 3

    def __init__(self):
        '''
        Constructor
        '''
        super().__init__(self._VERSION,self._LENGTH)
        self.reset()
        
    def parse(self,msg):
        try:
            m = struct.unpack('>4sHHBBBBHHxxHBBBB',msg[0:self._mlen])
            if m[0] != self.PREAMBLE: 
                raise('CONFFormatException',
                      'This is not an em configuration message')
            
            self._fid = m[1]
            self._svnver = m[2]
            self._conver = m[3]
            if self._conver != self._version: 
                raise Exception('CONFVersionException',
                                'This is not the proper version.')
            self._states = m[5]
            self._txcnt = m[6]
            self._systmt = m[7]
            self._samplc = m[9]
            self._thold = m[10]
            self._rxsc = m[11]
            self._axsc = m[12]
            self._bnsc = m[13]
        except Exception as e:
            self._error = True
            self.reset()
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))  
        
class confv5(conf):
    '''
    classdocs
    '''
    
    _LENGTH = 27
    _VERSION = 5

    def __init__(self):
        '''
        Constructor
        '''
        super().__init__(self._VERSION,self._LENGTH)
        self.reset()
    
    def checksum(self,byte_values):
        '''
        calculate a checksum value of a section of bytes
        '''
        total=0
        for b in byte_values: 
            total+=b
        return total & 255
    
    def parse(self,msg):
        self.reset()
        
        try:
            if msg[-1] != 10: 
                raise Exception('CONFEndOfLineException',
                                              'CONF Message does not end with a line feed')
            
            m = struct.unpack('>4s4B2H4B2HBH4B',msg[0:self._mlen])
            if m[0] != self.PREAMBLE: 
                raise('CONFFormatException',
                      "Did not begin with CONF")
            self._chksum = m[1]
            self._conver = m[2]
            if self._conver != self._version: 
                raise Exception('CONFVersionException',
                                'CONF version %d instead of CONF version %d'%(self._conver,self._version))
                
            if self.checksum(msg[5:-1]) != self._chksum:
                raise Exception('CONFVersionException',
                                'Checksum did not match: %d vs %d'%(self._chksum,self.checksum(msg[5:-1])))
            #check sum version
            self._chkver = m[3]
            self._sysid = m[4]
            self._fid = m[5]
            self._svnver = m[6]
            
            self._states = m[9]
            self._txcnt = m[10]
            self._systmt = m[11]
            self._samplt = m[12]
            self._samplc = m[14]
            self._thold = m[15]
            self._rxsc = m[16]
            self._axsc = m[17]
            self._bnsc = m[18]
            
            frmt='>%dH'%(self._bnsc)
            self._bins = list(struct.unpack(frmt,msg[self._mlen:self._mlen+2*self._bnsc]))
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))  


'''
Created on Apr 17, 2019

@author: reynolds
'''

from PyQt5.QtCore import pyqtSignal

from wrtdk.io.streamer.streamer import LoggingStreamer
from wrtdk.parser.nmea.nmea import gpgga, gprmc

import time,os,sys,struct
import numpy as np
from _ast import Try
from wrtdk.data.buffer.buffer import byte_buffer

class tcp_streamer(LoggingStreamer):
    '''
    a streamer for streaming and logging TCP data
    Based largely off of Randals Serial_streamer
    '''
    EOL = bytes([10])
    new_msg = pyqtSignal(list,'QString')
    new_timeout = pyqtSignal()

    def __init__(self,port=None):
        '''
        Constructor
        '''
        self.buffer=byte_buffer()
        super().__init__()
        self.port = port
        self._logging = False
        self.gga = gpgga()
        self.rmc = gprmc()
        self.mcu_s = 0
        self.mcu_ms = 0
        self.gpsid = struct.pack('4sss',b'$GPS',bytes(0),bytes(0))
        self.fid = np.uint16(0)
        self.p1 = np.uint16(1)
        
    def setMCU(self,mcu=0):
        ''' sets the mcu time to write to the file '''
        self.mcu_s = int(mcu)
        self.mcu_ms = int( (mcu - self.mcu_s) * 1000 )
        
    def run(self):
        if self.port is not None: self._running = True
        
        while self._running:
            try:
                #Read in the msg. might be more than 1
                msg,addr = self.port.read(1024)
                
            except Exception as e:
                #self.new_msg.emit([time.time(),])
                exc_type, _, exc_tb = sys.exc_info()
                fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                print('%s:%s in %s at %d'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno))
                self.new_timeout.emit()
                if exc_type.__name__ == 'AttributeError': time.sleep(1)
                #self.new_msg.emit(['$ERROR',time.time(),addr,np.nan],'ND',0,0)# send an error message
                continue
            # if an empty mesage is received then skip it
            if msg is None: continue
            
            if len(msg) < 1: 
                self.new_msg.emit(['$EMPTY',time.time(),addr,np.nan],'ND',0,0)# send an error message
                continue
            
                if msg[-1] is not 10: msg = msg + b'\n'# add a new line if necessary
                self.writer.write(wrapper)
            #add the new data to the buffer
            self.buffer.append(msg)
            
            # parse the messages
            m=self.parse()
            for val in m:
                if self.isLogging():
                    try:
                        string = val.decode().strip().encode()
                    except Exception as e:
                        exc_type, _, exc_tb = sys.exc_info()
                        fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                        print('%s:%s in %s at %d'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno))
                        continue
                    if(string[-1]!=b'\n'):
                        string+=b'\n'
                    wrapper = struct.pack('>6s4H2s5I%ds' % len(string),
                                                           b'$COUNT',0,0,0,self.fid,b'AG',
                                                           self.mcu_s,self.mcu_ms,0,len(string),0,string)
                    self.writer.write(wrapper)
                #m should be a split list of all the values. m[-1] should be the most recent
                self.new_msg.emit(m,'AG')
                self.fid+=self.p1
        try:
            self.port.close()
        except Exception as e:
            print('Error.',str(e))
        
    def parse(self):
        if self.buffer.is_empty(): return
        try:
            #split the buffer into the correct chunks
            temp=self.buffer.buffer
            split_list=temp.split(b'\n')
            
        except Exception as e:
            print('error',str(e),self.buffer)
            return
        i=0
        while b'' in split_list:
            split_list.remove(b'')
        
        self.buffer.clear()
        if(not split_list[-1].endswith(b'\r\x00')):
            self.buffer.append(split_list.pop())
        for i in range(len(split_list)):
            split_list[i]=split_list[i].strip(b'\r\x00')
        return split_list
    def is_eol(self,b):
        return b in self.EOL
        


'''
Created on Feb 14, 2019

@author: reynolds
'''

import time
import numpy as np

from PyQt5.QtCore import pyqtSignal,pyqtSlot

from wrtdk.io.streamer.streamer import LoggingStreamer

class fieldfox_streamer(LoggingStreamer):
    '''
    a class for stream data from fieldfox VNA
    '''
    
    new_ascan = pyqtSignal(list,str,int,int)

    def __init__(self,port=None):
        '''  constructor  '''
        super().__init__()
        self.port = port
        self._logging = False
        self._d = np.nan
        
    def run(self):
        ''' the aux sensor thread '''

        if self.port is not None: self._running = True
        
        while self._running:
            try:
                if self.port.measure():
                    self.new_ascan.emit(self.port.getData(),
                                        'AG',0,0)
                    
                    if self._logging: 
                    	self.port.write(self._file,self._odir,self._d,time.time())
            except Exception as e:
                print(str(e),'Error in aux streamer.')
                
        try:
            self.port.close()
        except Exception as e:
            print('Error.',str(e))
        
    def startLog(self, filename,odir='data',ftype='w+'):
        ''' starts the log '''
        try:
            self._file = filename
            self._odir = odir
            self.port.header(fn=self._file,odir=self._odir)
            self._logging = True
        except Exception as e:
            print('Error starting aux log file.',str(e))
            
    def isLogging(self):
        ''' returns whether the device is logging '''
        return self._logging
            
    def stopLog(self):
        ''' stops the aux log '''
        self._logging = False
    def setDistance(self,d=np.nan):
        ''' sets the distance '''
        self._d = d
        


'''
Created on Apr 8, 2020

@author: Reynolds
'''

import time

from wrtdk.io.sim.simulator import simulator

class apex_simulator(simulator):
    '''
    classdocs
    '''
    
    DOLLAR_SIGN = 36
    LITTLE_T = 116
    BIG_C = 67
    SOL = [BIG_C,LITTLE_T,DOLLAR_SIGN]
    DOLLAT_MSGS = [b'$GPGGA',b'$GNGGA',b'$IMUAP']

    def __init__(self):
        '''
        Constructor
        '''
        super().__init__()
        
    def _is_sol(self,b):
        return b in self.SOL
    
    def _is_eol(self,b):
        return b == 10
    
    def _is_pos_msg(self,buffer):
        return buffer in self.DOLLAT_MSGS
    
    def _is_conf_msg(self,buffer):
        return buffer == b'CONF'
        
    def read(self, filename):
        t0 = time.time()
        simulator.read(self, filename)
        
        if not self._error:
            with open(filename,'rb') as f:
                buffer = f.read()
                length = len(buffer)
                
                sol = []
                for i,b in enumerate(buffer):
                    if self._is_sol(b) and self._is_eol(buffer[i-1]):
                        sol.append(i)
                sol.append(length)
                
                i = 0
                while i < len(sol)-1:
                    start = sol[i]
                    end = sol[i+1]-1
                    
                    if buffer[start] == self.LITTLE_T:
                        if self._is_eol(buffer[end]):
                            self._msgs.append(buffer[start:end+1])
                            self._dt.append(0.001)
                            i+=1
                        else: sol.remove(start)
                    elif buffer[start] == self.DOLLAR_SIGN and length - start > 6:
                        if self._is_eol(buffer[end]) and self._is_pos_msg(buffer[start:start+6]):
                            self._msgs.append(buffer[start:end+1])
                            self._dt.append(0.001)
                            i += 1
                        else: sol.remove(start)
                    elif buffer[start] == self.BIG_C and length - start > 4:
                        if self._is_eol(buffer[end]) and self._is_conf_msg(buffer[start:start+4]): 
                            self._msgs.append(buffer[start:end+1])
                            self._dt.append(0.001)
                            i += 1
                        else: sol.remove(start)
                    else: sol.remove(start)
                self._dt = self._dt[1::] + [4]
                self._len = len(self._msgs)
                print('Done. %.2fs'%(time.time()-t0))
                return True
            print('Error. %.2fs'%(time.time()-t0))
            return False
        print('Error. %.2fs'%(time.time()-t0))
        return False


'''
Created on Nov 20, 2019

@author: reynolds
'''
import unittest
from wrtdk.parser.nmea.nmea import gprmc

class test_rmc(unittest.TestCase):


    def setUp(self):
        self.gprmc = b'$GPRMC,081836,A,3751.65,S,14507.36,E,000.0,360.0,130998,011.3,E*62'
        self.gnrmc = b'$GNRMC,081836,A,3751.65,S,14507.36,E,000.0,360.0,130998,011.3,E*62'
        self.rmc = gprmc()

    def tearDown(self):
        pass


    def test_gprmc_parse(self):
        self.rmc.parse(self.gprmc)
        self.assertFalse(self.rmc.hasErrored())
        
    def test_gnrmc_parse(self):
        self.rmc.parse(self.gnrmc)
        self.assertFalse(self.rmc.hasErrored())

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on Jan 20, 2020

@author: Reynolds
'''
import unittest
from wrtdk.parser.tfmag.mfam import mx3g

class Test(unittest.TestCase):

    def setUp(self):
        self.msg = b'\xea\x7f\x00\x04CA)<\x00\x04\x00\x04C\xf2\x99\xa8\x1c\r\x00\x009%\x00\x00\x00\x00\x00\x00'
        self.m = mx3g()

    def test_parse(self):
        self.m.parse(self.msg)
        
        self.assertFalse(self.m.hasErrored())
        
        data = self.m.getData()
        
        msg = self.m.get(m1=data[0],
                        m2=data[1],
                        v1=data[13],
                        v2=data[14],
                        s1=data[6],
                        s2=data[7],
                        count=data[9],
                        failed=data[12],
                        synch=data[11],
                        pps=data[10],
                        a0=data[15],
                        a1=data[16],
                        a2=data[17],
                        adc0=data[2],
                        adc1=data[3],
                        adc2=data[4],
                        temp=data[5])
        
        self.m.parse(msg)
        
        self.assertFalse(self.m.hasErrored())
        
        d = self.m.getData()
        
        #print(data)
        #print(d)
        
        self.assertEqual(data,d)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on Aug 17, 2018

@author: reynolds
'''
import time

class state(object):
    '''
    a class for representing the state. It alerts the user if the
    state changes
    '''
    
    def __init__(self):
        ''' constructor '''
        self.reset()
             
    def isDifferent(self,val=-1):
        ''' checks to see if the current value is different 
        and updates if the value is different
        '''
        if self._curr != val:
            self._curr = val
            return True
        return False
    
    def reset(self):
        self._curr = []
    
class updater(object):
    '''
    controls when to udpate sensor data based on a number of update samples
    '''
    
    def __init__(self,update=10):
        ''' constructor '''
        self.reset()
        self._update = update
        
    def update(self):
        ''' calculates whether to update or not '''
        self._n += 1
        if self._n > self._update:
            self.reset()
            return True
        return False
        
    def reset(self):
        self._n = 1
        
class refresher(object):
    '''
    a class for refreshing an event after a specific elapsed amount of time
    '''
    
    def __init__(self,latency=1.0):
        ''' constructor '''
        self.reset()
        self._latency = latency
        
    def _elapsed(self):
        ''' calculates elapsed  time '''
        return time.time() - self._time
        
    def update(self):
        ''' returns whether an update should occur '''
        if self._elapsed() > self._latency:
            self.reset()# reset the initial time
            return True# update
        return False# dont update
        
    def reset(self):
        ''' resets the initial time '''
        self._time = time.time()
        


'''
Created on Apr 3, 2020

@author: Reynolds
'''

import datetime
import numpy as np

from wrtdk.parser.em.em3d.em.em import emv2, emv1
from wrtdk.parser.em.em3d.conf.conf import confv5, confv3
from wrtdk.parser.nmea.nmea import gpgga
from wrtdk.io.read.file_reader import file_reader
from wrtdk.parser.imu.microstrain import lord3dm_cv5

class apex_reader(file_reader):
    '''
    a class for reading in white river technologies 
    APEX EM sensing system proprietary DAT files. It
    returns the data in a dictionary. The class is 
    compatible with all marine and land based EM systems
    utilizing the CONF message style data format, versions
    3 and 5.
    '''
    DOLLAR_SIGN = 36
    LITTLE_T = 116
    BIG_C = 67
    ID_LEN = 6
    DATE_FRMT = '%d %m %Y %H %M %S'
    DEFAULT = '*'
    
    SOL = [BIG_C,LITTLE_T,DOLLAR_SIGN]
    
    def __init__(self,version=1,em3d=True):
        '''
        Constructor
        '''
        super().__init__()
        self.set_version(version=version)
        self._gparse = gpgga()
        self._iparse = lord3dm_cv5()
        self._clen = 0
        self.set_3d(em3d)
        self._rxs = 0
        self._axs = 0
        self._offset = 3
    
    def set_version(self,version=1):
        '''
        sets the version number. currently, 1 or 2
        are the available version numbers and defaults
        to version 1
        '''
        if version == 2:
            self._eparse = emv2()
            self._cparse = confv5()
            self._version = 2
        else:
            self._eparse = emv1()
            self._cparse = confv3()
            self._version = 1
            
    def set_3d(self,em3d=True):
        '''
        sets the 3D boolean. This should only be false
        for EMPACT DD
        '''
        self._3d = em3d
        if self._3d: self._txs = 6
        else: self._txs = 1
                    
    def get_version(self):
        '''
        returns the version number
        '''
        return self._version
    
    def _is_sol(self,b):
        '''
        returns whether a byte is a start of line character
        '''
        return b in self.SOL
    
    def _is_eol(self,b):
        '''
        checks whether a byte is an end of line character 
        '''
        return b == 10
    
    def _set_length(self,rx=0,ax=0):
        '''
        Sets the number of axes, receivers and the
        number of messages to count
        '''
        self._axs = ax
        self._rxs = rx
        self._len = self._offset + self._txs*self._rxs*self._axs
        
    def _get_header(self,buffer):
        ''' 
        returns the header information
        '''
        h = {}
        h['time'] = self._get_datetime(self._get_line(buffer,b'ACQ TIME:'))
        h['julian_time'] = self._get_julian(h['time'])
        h['zulu_time'] = self._get_zulu(h['time'])
        h['mode'] = self._get_line(buffer,b'OPERATION MODE:')
        h['operator'] = self._get_line(buffer,b'OPERATOR:')
        h['project'] = self._get_line(buffer,b'PROJECT:')
        h['grid'] = self._get_line(buffer,b'GRID')
        h['line'] = self._get_line(buffer,b'LINE')
        h['equipment_version'] = self._get_line(buffer,b'EQUIPMENT VER:')
        h['equipment_sn'] = self._get_line(buffer,b'EQUIPMENT SN')
        h['software_ver'] = self._get_line(buffer,b'ACQ SOFTWARE VER:')
        h['software_path'] = self._get_line(buffer,b'ACQ SOFTWARE PATH:')
        h['sft_cal_path'] = self._get_line(buffer,b'SFT PATH:')
        h['orientation_sys'] = self._get_line(buffer,b'ORIENTATION SYS:')
        h['orientation_sys_offset'] = self._get_line(buffer,b'ORIENTATION SYS OFFSET:')
        h['spatial_reg_sys'] = self._get_line(buffer,b'SPATIAL REG SYS:')
        h['spatial_reg_sys_offset'] = self._get_line(buffer,b'SPATIAL REG SYS OFFSET:')
        h['ground_tx_assembly_h'] = self._get_line(buffer,b'TX ASSEMBLY HEIGHT:')
        h['tx_assembly_z_coil_h'] = self._get_line(buffer,b'Z COIL ABOVE TX ASSEMBLY HEIGHT:')
        
        return h
    
    def _get_line(self,buffer,string):
        '''
        Returns a line in the header based on the search string and
        buffer bytes object
        '''
        pos = buffer.find(string)
        if pos == -1: return '*'
        
        eol = buffer.find(b'\n',pos)
        if eol == -1: return '*'
        
        return buffer[pos+len(string):eol+1].decode().strip()
        
    def _get_zulu(self,dt):
        try:
            return dt.strftime('%Y-%m-%dT%H:%M:%S.%fZ')
        except:
            return self.DEFAULT
    
    def _get_julian(self,dt):
        try:
            tt = dt.timetuple()
            return '%04d%03d' % (tt.tm_year,tt.tm_yday)
        except:
            return self.DEFAULT
    
    def _get_datetime(self,line):
        '''
        parses the header timestamp into a datetime object
        '''
        try:
            return datetime.datetime.strptime(line,self.DATE_FRMT)
        except Exception as e:
            print('Error parsing timestamp %s'%str(e))
            return '*'
    
    def _get_conf(self,buffer):
        '''
        Parses the first CONF message to set the EM data.
        If no messages of the proper type are found, it 
        returns a false boolean for whether a CONF message
        has been found. True if it does find a correct CONF
        message
        '''
        pos = 0
        # cycle through all conf messages in the first is corrupt
        while pos < len(buffer):
            # find the first messade
            pos = buffer.find(b'CONF',pos)
            
            # find the end of the message
            i = 1# counter from the current position
            while i+pos < len(buffer):# cycle through the entire buffer
                # check if the current byte starts the next message
                if (self._is_sol(buffer[i+pos]) and 
                    self._is_eol(buffer[i+pos-1])): break# if true then break.
                i += 1# increment the counter
            
            # parse the message
            self._cparse.parse(buffer[pos:pos+i])
            if not self._cparse.hasErrored():# check for errors
                self._clen = self._cparse.get_length()
                self._eparse.set_bins(self._cparse.get_bin_count())# set the bins
                return True# message found
            # increment the position
            pos = pos+1
            
        print('No CONF message found.')
        return False
    
    def _get_sol(self,buffer):
        '''
        Returns the start of line characters for the bytes
        object buffer. The last element in the start of line
        list is the length of the buffer so the entire last
        message can be called. Additionally returns the message 
        count for all messages found.
        '''
        sol = []# hold start of line indices
        length = len(buffer)# length of buffer
        for i,b in enumerate(buffer):# enumerate buffer length
            if self._is_sol(b) and self._is_eol(buffer[i-1]): sol.append(i)# add index if start of line
        sol.append(length)# add the end of the file
            
        # trim and count the messages
        count = [0] * self._eparse.get_output_length(txs=6,# count the messages
                                                     rxs=self._cparse.get_rxs(),
                                                     axs=self._cparse.get_axes())
        start = 0# start msg index
        end = 0# end msg index
        i = 0# position in sol indices
        while i < len(sol)-1:
            start = sol[i]# calculate position
            end = sol[i+1]-1# calculate end
            
            # sot by message start character
            if buffer[start] == self.LITTLE_T:# em
                if (self._is_eol(buffer[end])):# confirm em message
                    try:
                        count[self._eparse.get_em_index(buffer[start+2],
                                                    self._cparse.get_rxs(),
                                                    self._cparse.get_axes())] += 1
                    except Exception as e:
                        print(str(e),buffer[start-1:end+1])
                        return (sol,count)             
                    i+=1# increase position in sol list
                else: 
                    print('removed:',end-start,start,'/',length,buffer[start:end])
                    sol.remove(start)# remove item from list. no duplicates possible
            elif (buffer[start] == self.DOLLAR_SIGN and 
                  length - start > self.ID_LEN):# position/attitude
                if ((buffer[start:start+self.ID_LEN] == b'$GPGGA' or
                    buffer[start:start+self.ID_LEN] == b'$GNGGA') and 
                    self._is_eol(buffer[end])):# confirm
                    count[self._eparse.get_pos_index()] += 1
                    i+=1# increase position in sol list
                elif (buffer[start:start+self.ID_LEN] == b'$IMUAP' and 
                        self._is_eol(buffer[end]) ):
                    count[self._eparse.get_imu_index()] += 1
                    i+=1# increase position in sol list
                else: sol.remove(start)# remove item from list. no duplicates possible
            elif (buffer[start] == self.BIG_C and 
                  length - start > 4):# em configuration
                if (self._is_eol(buffer[end])  and 
                    buffer[start:start+4] == b'CONF'):
                    count[self._eparse.get_conf_index()] += 1
                    i+=1# increase position in sol list
                else: sol.remove(start)# remove item from list. no duplicates possible
            else: sol.remove(start)# remove item from list. no duplicates possible
        return (sol, count)
    
    def _get_axis(self,ax=0):
        '''
        returns the sensing axis from the integer
        '''
        if ax == 0: return 'x'
        elif ax == 1: return 'y'
        elif ax == 2: return 'z'
        else: return 'w'
        
    def _get_tx(self,tx=0):
        ''' 
        returns the Tx axis from the integer
        '''
        if tx == 0: return 'x+'
        elif tx == 1: return 'A'
        elif tx == 2: return 'y+'
        elif tx == 3: return 'B'
        elif tx == 4: return 'z+'
        else: return 'C'
            
    def _read(self,file):
        '''
        Reads the file into a byte array and then
        parses the individual messages if the file
        is the correct type. It returns a dictionary
        filled with the data: EM, CONF, HEADER, IMU
        and GNSS
        '''
        d = {}
        with open(file,'rb') as f:
            b = f.read()# read all the bytes
            if not self._get_conf(b): return d# get the first conf message
            
            # intialize the dictionary
            d['HEADER'] = self._get_header(b)# header info
            sol,c = self._get_sol(b)# get sol indices and count messages
            d['HEADER']['message_count'] = len(sol)-1
            d['CONF'] = {'count':    np.arange(c[self._eparse.get_conf_index()]),
                         'fiducial': np.zeros( (c[self._eparse.get_conf_index()]) ),
                         'number':   np.zeros( (c[self._eparse.get_conf_index()]) ),
                         'data':     np.zeros( (c[self._eparse.get_conf_index()],
                                                len(self._cparse.getData())) )}# conf messages
            d['GNSS'] = {'count':    np.arange(c[self._eparse.get_pos_index()]),
                         'fiducial': np.zeros( (c[self._eparse.get_pos_index()]) ),
                         'number':   np.zeros( (c[self._eparse.get_pos_index()]) ),
                         'data':     np.zeros( (c[self._eparse.get_pos_index()],
                                                len(self._gparse.getData())) )}# position data
            d['IMU'] = {'count':    np.arange(c[self._eparse.get_imu_index()]),
                        'fiducial': np.zeros( (c[self._eparse.get_imu_index()]) ),
                        'number':   np.zeros( (c[self._eparse.get_imu_index()]) ),
                        'data':     np.zeros( (c[self._eparse.get_imu_index()],
                                               3)) }# imu data
            d['EM'] = {}# em data
            for i in range(self._txs):# cycle through all transmitters
                tx = self._get_tx(i)
                d['EM'][tx] = {}
                start = self._eparse.get_em_index((i,0,0),
                                                  self._cparse.get_rxs(),
                                                  self._cparse.get_axes())
                stop = self._eparse.get_em_index((i,self._cparse.get_rxs()-1,self._cparse.get_axes()-1),
                                                 self._cparse.get_rxs(),
                                                 self._cparse.get_axes())
                rows = max(c[start:stop+1])
                d['EM'][tx] = {'count':    np.arange( rows ),
                               'fiducial': np.zeros( (rows,self._cparse.get_rxs()*self._cparse.get_axes()) ),
                               'number':   np.zeros( (rows,self._cparse.get_rxs()*self._cparse.get_axes()) ),
                               'data':     np.zeros( (rows,self._cparse.get_rxs()*self._cparse.get_axes(),
                                                      self._cparse.get_bin_count()) )}
                        
            # populate the data fields
            c = [0] * len(c)# reinitialize counter
            index,tx,loc,msgid = 0,'',0,b''
            for i in range(len(sol)-1):
                if b[sol[i]] == self.LITTLE_T:
                    self._eparse.parse(b[sol[i]:sol[i+1]])
                    if not self._eparse.hasErrored():
                        index = self._eparse.get_em_index(b[sol[i]+self._eparse.get_id_index()],
                                                          self._cparse.get_rxs(),
                                                          self._cparse.get_axes())
                        tx = self._get_tx(self._eparse.get_tx())
                        loc = self._eparse.get_rx()*self._cparse.get_axes()+self._eparse.get_axis()
                        d['EM'][tx]['data'][c[index],loc,:] = self._eparse.get_decay()
                        d['EM'][tx]['number'][c[index],loc] = i
                        d['EM'][tx]['fiducial'][c[index],loc] = self._eparse.get_fiducial()
                        #print(self._eparse)
                        c[index] += 1
                elif b[sol[i]] == self.DOLLAR_SIGN:
                    msgid = b[sol[i]:sol[i]+self.ID_LEN]
                    if (msgid == b'$GPGGA' or
                        msgid == b'$GNGGA') :
                        self._gparse.parse(b[sol[i]:sol[i+1]])
                        if not self._gparse.hasErrored():
                            index = self._eparse.get_pos_index()
                            d['GNSS']['number'][c[index]] = i
                            d['GNSS']['data'][c[index],:] = self._gparse.getData()
                            c[index] += 1
                    elif msgid == b'$IMUAP':
                        self._iparse.parse(b[sol[i]+self.ID_LEN:sol[i+1]-1])
                        if not self._iparse.hasErrored():
                            index = self._eparse.get_imu_index()
                            d['IMU']['data'][c[index],:] = self._iparse.get_euler()
                            c[index] += 1
                elif b[sol[i]] == self.BIG_C:
                    self._cparse.parse(b[sol[i]:sol[i+1]])
                    if not self._cparse.hasErrored():
                        index = self._eparse.get_conf_index()
                        d['CONF']['data'][c[index],:] = self._cparse.getData()
                        d['CONF']['number'][c[index]] = i
                        d['CONF']['fiducial'][c[index]] = self._cparse.get_fiducial()
                        c[index] += 1   
        return d


'''
Created on Nov 20, 2019

@author: reynolds
'''
import unittest
from wrtdk.parser.em.em3d.em.em import emv1


class test_emv1(unittest.TestCase):


    def setUp(self):
        self.msg = b't\xd5L\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\n'
        
    def tearDown(self):
        pass

    def test_parse(self):
        em = emv1()
        em.parse(self.msg)
        self.assertTrue(em.hasErrored())
        
        em.set_bins(16)
        
        em.parse(self.msg)
        
        self.assertFalse(em.hasErrored())
        
if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on Nov 5, 2019

@author: reynolds
'''

import unittest
from wrtdk.data.buffer.buffer import ring_buffer

class test_ring_buffer(unittest.TestCase):
    '''
    classdocs
    '''
    
    def test_length(self):
        length = 100
        b = ring_buffer(length)
        self.assertEqual(b.get_length(),length)
        
    def test_set_length(self):
        length = 10
        b = ring_buffer()
        b.set_length(length)
        self.assertEqual(b.get_length(),length)
        
    def test_is_full(self):
        length = 1
        b = ring_buffer(length)
        before = b.is_full()
        b.append([0])
        self.assertNotEqual(before,b.is_full())
        
    def test_is_empty(self):
        b = ring_buffer(10)
        empty = b.is_empty()
        b.append([1])
        not_empty = b.is_empty()
        self.assertNotEqual(empty,not_empty)
        
    def test_get_count(self):
        app = [1]
        b = ring_buffer(10)
        b.append([1])
        self.assertEqual(b.get_count(),len(app))
        
    def test_has_space(self):
        b = ring_buffer(10)
        under,over = b.has_space(b.get_length()-1),b.has_space(b.get_length()+1)
        self.assertNotEqual(under,over)
    
    def test_append_get(self):
        b = ring_buffer(10)
        x = list(range(0,b.get_length()))
        b.append(x)
        x0 = b.get(1)
        self.assertEqual(x[0:1],x0)
        n = [100]
        b.append(n)
        xn = b.get(b.get_length())
        self.assertEqual(x[1::] + n,xn)
        
if __name__ == '__main__':
    unittest.main()
        


'''
Created on Jan 31, 2019

@author: reynolds
'''

from wrtdk.parser.parser import parser
import sys, os, re, utm
import numpy as np

class gcs(object):
    ''' handles gcs coordinates '''
    
    def dm2dd(self,dm,direction):
        '''  Converts a geographic coordiante given in "degres/minutes" dddmm.mmmm
        format (ie, "12319.943281" = 123 degrees, 19.953281 minutes) to a signed
        decimal (python float) format '''
        # '12319.943281'
        if not dm or dm == '0': return 0.
        d, m = re.match(r'^(\d+)(\d\d\.\d+)$', dm).groups()
        value = float(d) + float(m) / 60
        
        if direction == 'N' or direction == 'E': return value
        elif direction == 'S' or direction == 'W': return -value
        else: return np.NaN
        
    def dd2dm(self,dd=0,direction='latitude'):
        d,m = divmod(abs(dd),1)
        m *= 60
        
        if direction is 'latitude':
            if dd >= 0: return (d,m,'N')
            else: return (d,m,'S')
        else:
            if dd >= 0: return (d,m,'E')
            else: return (d,m,'W')
            
class utc_time(object):
    
    def s2dms(self,s=0):
        d,m = divmod(s,3600)
        m,s = divmod(m,60)
        return (d,m,s)
        
    def dms2s(self,dms):
        return float(dms[0:2])*3600 + float(dms[2:4])*60 + float(dms[4:])
    
class nmea_parser(parser):
    
    def __init__(self):
        super().__init__()
        
    def checksum(self,msg=b''):
        _sum = 0
        for b in msg: _sum ^=b
        return (hex(_sum).split('x')[-1].upper().encode(),_sum)

class gpgga(nmea_parser):
    ''' $GPGGA sentence parser '''
    
    BYTES = type(bytes())
    
    def __init__(self):
        ''' Constructor '''
        super().__init__()
        self.reset()
        self._gcs = gcs()
        self._utc = utc_time()
    
    def reset(self):
        ''' resets the parser '''
        self._time = self._nan()
        self._timestamp = '00:00:00.00'
        self._lat = self._nan()
        self._lon = self._nan()
        self._fix = self._minus1()
        self._n = self._minus1()
        self._easting = self._nan()
        self._northing = self._nan()
        self._region = self._minus1()
        self._zone = ''
        self._dop = self._nan()
        self._alt = self._nan()
        self._geoid = self._nan()
        
    def get(self,talker_id=b'$GPGGA',time=0,latitude=0,longitude=0,
            fix=0,nsat=0,dop=100,altitude=0,geoid=0,t_dgps=None):
        
        msg = b''
        
        if type(talker_id) != self.BYTES: talker_id = talker_id.encode()
        msg += talker_id
        
        utc_d,utc_m,utc_s = self._utc.s2dms(time)
        msg += b',%02d%02d%02.2f'%(utc_d,utc_m,utc_s)
        
        lat_d,lat_m,lat_dir = self._gcs.dd2dm(latitude,'latitude')
        msg += b',%02d%010.7f,%s' % (lat_d,lat_m,lat_dir.encode())
        
        lon_d,lon_m,lon_dir = self._gcs.dd2dm(longitude,'longitude')
        msg += b',%03d%010.7f,%s'%(lon_d,lon_m,lon_dir.encode())
        
        msg += b',%d,%02d,%.1f,%.1f,M,%.1f,M,'%(fix,nsat,dop,
                                                altitude,geoid)
        
        if t_dgps is None: msg += b',*'
        else: msg += b'%.2f,*'
        
        return msg + self.checksum(msg[1:-1])[0]
        #b'$GPGGA,123519,4807.038,N,01131.000,E,1,08,0.9,545.4,M,46.9,M,,*47'
        
    def parse(self,msg):
        ''' parses the message in ascii or binary '''
        try:
            if type(msg) == self.BYTES: msg = msg.decode()
            if (not msg.startswith('$GPGGA') and 
                not msg.startswith('$GNGGA') and 
                not msg.startswith('$GLGGA')):
                print('ImproperGPGGAMessageError.')     
            string = msg.split(',')
            self._timestamp = '%s:%s:%s' % (string[1][0:2],
                                            string[1][2:4],
                                            string[1][4:])
            self._time = float(string[1][0:2])*3600 + float(string[1][2:4])*60 + float(string[1][4:])
            self._lat = self._gcs.dm2dd(string[2],string[3])
            self._lon = self._gcs.dm2dd(string[4],string[5])
            [self._easting,self._northing,self._region,self._zone] = utm.from_latlon(self._lat,
                                                                                     self._lon)
            self._fix = int(string[6])
            self._n = int(string[7])
            self._dop = float(string[8])
            self._alt = float(string[9])
            self._geoid = float(string[11])
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
            
    def getData(self):
        ''' returns the data 
        0) time in seconds
        1) timestamp in HH:MM:SS.ss
        2) latitude in decimal degrees
        3) longitude in decimal degrees
        4) number of satellites
        5) GPS fix quality
        6) Dilution of precision
        7) UTM easting in m
        8) UTM northing in m
        9) UTM Region
        10) UTM zone
        11) Height above sea level in m
        12) Height above geoid in m'''
        return [self._time,self._timestamp,
                self._lat,self._lon,
                self._n,self._fix,self._dop,
                self._easting,self._northing,self._region,self._zone,
                self._alt,self._geoid]
        
class gprmc(parser):
    ''' parses the gprms nmea messages '''
    
    BYTES = type(bytes())
    
    def __init__(self):
        ''' constructor '''
        super().__init__()
        self.reset()
        self._gcs = gcs()
        
    def reset(self):
        ''' resets the parser '''
        self._time = self._nan()
        self._timestamp = '00:00:00.00'
        self._lat = self._nan()
        self._lon = self._nan()
        self._easting = self._nan()
        self._northing = self._nan()
        self._region = self._minus1()
        self._zone = ''
        self._warn = ''
        self._s = self._nan()
        self._c = self._nan()
        self._date = ''
        self._var = self._nan()
        self._var_dir = ''
        
    def parse(self,msg):
        ''' parses the message '''
        try:
            if type(msg) == self.BYTES: msg = msg.decode()
            if (not msg.startswith('$GPRMC') and 
                not msg.startswith('$GNRMC') and 
                not msg.startswith('$GLRMC')):
                print('ImproperGPRMCMessageError.')     
            string = msg.split(',')
            self._timestamp = '%s:%s:%s' % (string[1][0:2],
                                            string[1][2:4],
                                            string[1][4:])
            self._time = float(string[1][0:2])*3600 + float(string[1][2:4])*60 + float(string[1][4:])
            self._warn = string[2]
            self._lat = self._gcs.dm2dd(string[3],string[4])
            self._lon = self._gcs.dm2dd(string[5],string[6])
            self._s = float(string[7])
            self._c = string[8]
            self._date = string[9]
            self._var = string[10]
            self._var_dir = string[11]
            
            [self._easting,self._northing,self._region,self._zone] = utm.from_latlon(self._lat,
                                                                                     self._lon)
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
        
    def getData(self):
        ''' returns the data 
        0) time in seconds
        1) timestamp
        2) waring A=OK, V=warning
        3) latitude in decimal degrees
        4) longitude in decimal degrees
        5) utm easting in m
        6) utm northing in m
        7) utm region
        8) utm zone
        9) speed in knots
        10) true course
        11) date stamp
        12) variation
        13) east/west '''
        return [self._time,self._timestamp,self._warn,
                self._lat,self._lon,
                self._easting,self._northing,self._region,self._zone,
                self._s,self._c,self._date,self._var,self._var_dir]    


'''
Created on Feb 14, 2019

@author: reynolds
'''

import sys, os

from PyQt5.Qt import QThread
from wrtdk.io.writer import FileLogger

class LoggingStreamer(QThread):
    
    def __init__(self,port=None,running=False,debug=False,dt=0.1):
        ''' constructor '''
        super().__init__()
        self._running = running
        self._debug = debug
        self.port = port
        self.writer = FileLogger()
        if not self._debug: self._running = True
    
    def stop(self):
        '''  stops the thread  '''
        self._running = False
        self._debug = False
    
    def startLog(self,filename,ftype='w+'):
        '''  starts the log  '''
        try:
            self.writer.open(filename,ftype=ftype)
        except Exception as e:
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno))
            
    def stopLog(self):
        '''  stops logging  '''
        try:
            self.writer.close()
        except Exception as e:
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno))
        
    def isLogging(self):
        ''' returns whether the file writer is currently logging '''
        return self.writer.isLogging()
    
    def set_mode(self):
        ''' sets the mode of the streamer '''
        if not self._debug: self._running = True


'''
Created on Aug 17, 2018

@author: reynolds
'''

from PyQt5 import QtCore, QtGui
from PyQt5.QtWidgets import QComboBox, QWidget, QLabel, QPushButton, QLineEdit, QGridLayout, QFrame,\
    QListView
from PyQt5.QtGui import QFont
from PyQt5.Qt import Qt


class wrt_widget(QFrame):
    ''' allows for widgets to use commonly used WRT widget functions '''
    
    def __init__(self,parent=None):
        QFrame.__init__(self,parent)
        ''' constructor '''
    
    def _setState(self,widget,state):
        ''' sets the state of the qmag for the MAD Unit because
        John-Mike decided to change the firmware to something
        stupid at the last minute '''
        if state == '' or state == None:
            # if the state is nothing or the state is the same then return
            return
        elif state =='AG' or state == b'\x00\x00'.decode():
            # set the all good state
            widget.setStyleSheet('background-color: green')
        elif state == 'TO':
            # set the state to Timed Out
            widget.setStyleSheet('background-color: orange')
        elif state == 'NC':
            widget.setStyleSheet('background-color: yellow')
        else:
            # set everything else as bad
            widget.setStyleSheet('background-color: red')
        # set the text
        widget.setText(state)

class gpr_widget(wrt_widget):
    '''
    a class for plotting the gpr data
    '''
    
    LINES = 1
    
    def __init__(self,parent=None,timeout=2000):
        ''' constructor '''
        super().__init__(parent)
        self.timeout = timeout
        self.plot = WaterfallWidget(self,
                                    parameters=PlotParameters('A Scan','Amplitude','Time'),
                                    lines=1)
        self.count = self._getQLineEdit('',True)
        self.status = self._getQLineEdit('',readonly=True)
        
        l = QGridLayout()
        l.addWidget(self._getLabel('GPR',16,Qt.AlignCenter),0,0,1,4)
        l.addWidget(self._getLabel('Count:',10),1,0,1,1)
        l.addWidget(self.count,1,1,1,1)
        l.addWidget(self._getLabel('Status:',10,Qt.AlignCenter),1,2,1,1)
        l.addWidget(self.status)
        l.addWidget(self.plot,2,0,1,4)
        
        self.setLayout(l)
        self.setFrameStyle(QFrame.StyledPanel | QFrame.Plain)
        
        self.state = state()
        self.timer = QTimer()
        self.timer.timeout.connect(lambda: self.setStatus('NC'))
        
    def setStarted(self):
        ''' sets the display started '''
        self.timer.start(self.timeout)
    
    def setStopped(self):
        ''' resets the display '''
        self.timer.stop()
        self.count.setText('')
        self.status.setText('')
        self.status.setStyleSheet('')
        
    def _getLabel(self,text='',size=8,align=Qt.AlignRight|Qt.AlignVCenter):
        ''' sets the label font and size '''
        lbl = QLabel(text)
        lbl.setFont(QtGui.QFont('Times',size,QtGui.QFont.Bold))
        lbl.setAlignment(align)
        return lbl
    
    def _getQLineEdit(self,text='',readonly=True):
        ''' sets the line edit '''
        txt = QLineEdit()
        txt.setReadOnly(readonly)
        return txt
    
    def setCount(self,count=0):
        ''' sets the count '''
        self.count.setText('%.0f' % count)
    
    def setStatus(self,status):
        ''' do nothing '''
        self.timer.stop()
        self.timer.start(self.timeout)
        if self.state.isDifferent(status):
            self._setState(self.status,status)
    
    def setStrength(self,strength=0):
        ''' sets the strength '''
        return

class IMUWidget(QFrame):
    '''
    a class for displaying IMU euler angles
    '''
    
    def __init__(self,parent=None):
        ''' constructor '''
        QFrame.__init__(self,parent)
        
        # create the widgets
        self.roll = self._getQLineEdit()
        self.pitch = self._getQLineEdit()
        self.yaw = self._getQLineEdit()
        self.temperature = self._getQLineEdit()
        
        # layout the grid
        l = QGridLayout()
        l.addWidget(self._getLabel('IMU',10,Qt.AlignCenter),0,0,1,2)
        l.addWidget(self._getLabel('Roll:'),1,0)
        l.addWidget(self.roll,1,1)
        l.addWidget(self._getLabel('Pitch:'),2,0)
        l.addWidget(self.pitch,2,1)
        l.addWidget(self._getLabel('Yaw:'),3,0)
        l.addWidget(self.yaw,3,1)
        l.addWidget(self._getLabel('T (C):'),4,0)
        l.addWidget(self.temperature,4,1)
        l.addItem(QSpacerItem(10,10,QSizePolicy.Expanding,QSizePolicy.Expanding),5,0,1,2)
        l.setSpacing(10)
        self.setLayout(l)
        self.setFrameStyle(QFrame.StyledPanel | QFrame.Plain)
        
    def _getLabel(self,text='',size=8,align=Qt.AlignRight|Qt.AlignVCenter):
        ''' sets the label font and size '''
        lbl = QLabel(text)
        lbl.setFont(QtGui.QFont('Times',size,QtGui.QFont.Bold))
        lbl.setAlignment(align)
        return lbl
        
    def _getQLineEdit(self,text='',readonly=True):
        ''' sets the line edit '''
        txt = QLineEdit()
        txt.setReadOnly(readonly)
        return txt

class OutgoingWidget(QFrame):
    ''' allows the user to send commands to the sensor '''
    
    def __init__(self,parent=None):
        ''' constructor '''
        QFrame.__init__(self,parent)
        
        # create widgets
        font = QtGui.QFont('Times',12,QtGui.QFont.Bold)
        self.send = FHButtonInputWidget(self,'SEND','',spacing=10)
        self.send.setFont(font)
        self.startLog = QPushButton('SD LOG')
        #self.startLog.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.stopLog = QPushButton('STOP')
        #self.stopLog.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.startStream = QPushButton('STREAM')
        #self.startStream.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.stopStream = QPushButton('END')
        #self.stopStream.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        btns = QWidget(self)
        lb = QGridLayout()
        lb.addWidget(self.startLog,0,0,1,1)
        lb.addWidget(self.startStream,1,0,1,1)
        lb.addWidget(self.stopLog,0,1,1,1)
        lb.addWidget(self.stopStream,1,1,1,1)
        lb.setSpacing(0)
        btns.setLayout(lb)
        btns.setContentsMargins(0, 0, 0, 0)
        
        # layout widget
        l = QGridLayout()
        l.addWidget(btns,0,0,1,1)
        l.addWidget(self.send,0,1,1,1)
        l.setSpacing(10)
        self.setLayout(l)
        self.setContentsMargins(0, 0, 0, 0)
        self.setEnabled(False)
        
    def setEnabled(self,enabled=False):
        ''' sets the enabled state of the widget '''
        self.startLog.setEnabled(enabled)
        self.stopLog.setEnabled(enabled)
        self.startStream.setEnabled(enabled)
        self.stopStream.setEnabled(enabled)
        self.send.setEnabled(enabled)
        
class SoftwareLED(QWidget):
    ''' a class for a software LED '''
    
    def __init__(self,parent=None,text='',font=QFont('Times',12),align=Qt.AlignCenter):
        ''' constructor '''
        QWidget.__init__(self,parent)
        self.label = QLabel(text)
        self.label.setFrameShape(QFrame.Panel)
        self.label.setFrameShadow(QFrame.Raised)
        self.label.setLineWidth(2)
        self.label.setFont(font)
        self.label.setAlignment(align)
        l = QGridLayout()
        l.addWidget(self.label)
        self.setLayout(l)
        
    def setGood(self):
        ''' sets a good status '''
        self.label.setStyleSheet('background-color:green')
        
    def setWarning(self):
        ''' sets a warning status '''
        self.label.setStyleSheet('background-color:orange')
        
    def setBad(self):
        ''' sets a bad status '''
        self.label.setStyleSheet('background-color:red')

    def setState(self,state=''):
        self.label.setStyleSheet(state)
        
    def reset(self):
        ''' resets the led to the original state of transparent '''
        self.label.setStyleSheet('')

class CheckableComboBox(QComboBox):
    def __init__(self, parent = None):
        super(CheckableComboBox, self).__init__(parent)
        self.setView(QListView(self))
        self.view().pressed.connect(self.handleItemPressed)
        self.setModel(QtGui.QStandardItemModel(self))
        self._changed = False

    def handleItemPressed(self, index):
        item = self.model().itemFromIndex(index)
        if item.checkState() == QtCore.Qt.Checked and len(self.getCheckedItems())!=1:
            item.setCheckState(QtCore.Qt.Unchecked)
        else:
            item.setCheckState(QtCore.Qt.Checked)
        self._changed = True
            
    def getItems(self):
        return [self.itemText(i) for i in range(self.count())]
    
    def setIndexChecked(self,index,checked=True):
        item = self.model().item(index,self.modelColumn())
        if checked: item.setCheckState(QtCore.Qt.Checked)
        else: item.setCheckState(QtCore.Qt.Unchecked)
        
    def hidePopup(self):
        if not self._changed:
            super(CheckableComboBox, self).hidePopup()
        self._changed = False
        
    def itemChecked(self, index):
        item = self.model().item(index, self.modelColumn())
        return item.checkState() == QtCore.Qt.Checked

    def getCheckedItems(self):
        checkedItems = []
        for index in range(self.count()):
            item = self.model().item(index,self.modelColumn())
            if item.checkState() == QtCore.Qt.Checked:
                checkedItems.append(item.text())
        return checkedItems
class InputWidget(QWidget):
    ''' generic input widget with a label '''
    
    def __init__(self,parent=None,text=[],widget=[],spacing=10,row=[0,0],col=[0,1]):
        ''' constructor default layout is horizontal '''
        QWidget.__init__(self,parent)

        # set the text and input widgets
        self.text = text
        #self.text.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.widget = widget
        #self.widget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

        # layout the widget
        l = QGridLayout()
        l.setSpacing(spacing)
        l.addWidget(self.text,row[0],row[0],1,1)
        l.addWidget(self.widget,row[1],col[1],1,1)
        self.setLayout(l)
    
    def setLabelAlignment(self,alignment):
        ''' sets the label alignment '''
        self.text.setAlignment(alignment)
        
    def setWidgetAlignment(self,alignment):
        ''' sets the widget alignment '''
        self.widget.setAlignment(alignment)
        
    def setLabelFont(self,font):
        ''' sets the label font '''
        self.text.setFont(font)
        
    def setWidgetFont(self,font):
        ''' sets the widget font '''
        self.widget.setFont(font)
        
    def setAlignment(self,alignment):
        ''' sets the alignment '''
        self.setLabelAlignment(alignment)
        self.setLabelAlignment(alignment)
        
    def setFont(self,font):
        ''' sets the label font '''
        self.setLabelFont(font)
        self.setWidgetFont(font)
    
    def setEnabled(self,enabled=False):
        ''' sets the ui input enabled state '''
        self.widget.setEnabled(enabled)
        
    def setReadOnly(self,readOnly):
        ''' sets read only '''
        self.widget.setReadOnly(readOnly)

class LabelledInputWidget(InputWidget):
    ''' labelled input widget. Abstract '''
    
    def __init__(self,parent=None,label='',default='',spacing=10,row=[0,0],col=[0,1]):
        ''' constructor '''
        l = QLabel(label)
        t = QLineEdit(default)
        super().__init__(parent,l,t,spacing,row,col)

    
    def getLabel(self):
        ''' returns the labelled text '''
        pass
    
    def setLabel(self,label=''):
        ''' sets the label text '''
        pass

    def getText(self):
        ''' returns the line edit text '''
        return self.widget.text()

    def setText(self,text=''):
        ''' sets the line edit text '''
        self.widget.setText(text)
        
    def setInputColor(self,color='white'):
        ''' sets the color of the widget '''
        self.widget.setStyleSheet('background-color: ' + color)
        
    def setStyleSheet(self,style):
        ''' sets the style of the widget '''
        self.text.setStyleSheet(style)
        self.widget.setStyleSheet(style)
        
    def setReadOnly(self,readonly):
        ''' sets the read only status of the widget '''
        self.widget.setReadOnly(readonly)

class LabelledComboWidget(InputWidget):
    ''' Labelled combo widget '''
    
    def __init__(self,parent=None,label='',items=[],spacing=10,row=[0,0],col=[0,1]):
        ''' constructor '''
        c = QComboBox()
        c.addItems(items)
        l = QLabel(label)
        super().__init__(parent,l,c,spacing,row,col)

    def setItems(self,items=[]):
        ''' sets the items '''
        for s in items:
            self.widget.addItem(s)

    def getSelected(self):
        ''' returns the selected items '''
        return str(self.widget.currentText())

class ButtonInputWidget(InputWidget):
    ''' button input widget '''
    
    def __init__(self,parent=None,label='',default='',spacing=10,row=[0,0],col=[0,1]):
        ''' constructor '''
        b = QPushButton(label)
        t = QLineEdit(default)
        super().__init__(parent,t,b,spacing,row,col)

    def getText(self):
        ''' returns the text from the line edit '''
        return self.text.text()

    def setButtonColor(self,color):
        ''' sets the button color '''
        self.widget.setStyleSheet("background-color: " + color)

    def setButtonText(self,text=''):
        ''' sets the text of the button '''
        self.widget.setText(text)
        
    def setReadOnly(self,readonly):
        ''' sets the line edit read only status '''
        self.text.setReadOnly(readonly)

class FHLabelledInputWidget(LabelledInputWidget):
    ''' Forward horizontal input widget '''
    
    def __init__(self,parent=None,label='',default='',spacing=10):
        ''' constructor '''
        super().__init__(parent,label,default,spacing,[0,0],[0,1])

class RHLabelledInputWidget(LabelledInputWidget):
    ''' reverse horizontal labelled input widget '''
    
    def __init__(self,parent=None,label='',default='',spacing=10):
        ''' constructor '''
        super().__init__(parent,label,default,spacing,[0,0],[1,0])

class FVLabelledInputWidget(LabelledInputWidget):
    ''' Forward versitcal labelled input widget '''
    
    def __init__(self,parent=None,label='',default='',spacing=10):
        ''' constructor '''
        super().__init__(parent,label,default,spacing,[0,1],[0,0])

class RVLabelledInputWidget(LabelledInputWidget):
    ''' Reverse vertical labelled input widget '''
    
    def __init__(self,parent=None,label='',default='',spacing=10):
        ''' constructor '''
        super().__init__(parent,label,default,spacing,[1,0],[0,0])

class FHButtonInputWidget(ButtonInputWidget):
    ''' Forward horizontal button input widget '''
    
    def __init__(self,parent=None,label='',default='',spacing=10):
        ''' constructor '''
        super().__init__(parent,label,default,spacing,[0,0],[0,1])

class RHButtonInputWidget(ButtonInputWidget):
    ''' Reverse horizontal button input widget '''
    
    def __init__(self,parent=None,label='',default='',spacing=10):
        ''' constructor '''
        super().__init__(parent,label,default,spacing,[0,0],[1,0])

class FVButtonInputWidget(ButtonInputWidget):
    ''' Fowrard horizontal button input widget '''
    
    def __init__(self,parent=None,label='',default='',spacing=10):
        ''' constructor '''
        super().__init__(parent,label,default,spacing,[0,1],[0,0])

class RVButtonInputWidget(ButtonInputWidget):
    ''' Reverse vertical input widget '''
    
    def __init__(self,parent=None,label='',default='',spacing=10):
        ''' constructor '''
        super().__init__(parent,label,default,spacing,[1,0],[0,0])

class FHLabelledComboWidget(LabelledComboWidget):
    ''' Forward horizontal labelled combo widget '''
    
    def __init__(self,parent=None,label='',items=[],spacing=10):
        ''' constructor '''
        super().__init__(parent,label,items,spacing,[0,0],[0,1])

class RHLabelledComboWidget(LabelledComboWidget):
    '''Reverse horizontal labelled combo widget '''
    
    def __init__(self,parent=None,label='',items=[],spacing=10):
        ''' constructor '''
        super().__init__(parent,label,items,spacing,[0,0],[1,0])

class FVLabelledComboWidget(LabelledComboWidget):
    ''' Forward vertical labelled combo widget '''
    
    def __init__(self,parent=None,label='',items=[],spacing=10):
        ''' constructor '''
        super().__init__(parent,label,items,spacing,[0,1],[0,0])

class RVLabelledComboWidget(LabelledComboWidget):
    ''' a rear vertical combo widget '''
    
    def __init__(self,parent=None,label='',items=[],spacing=10):
        ''' constructor '''
        super().__init__(parent,label,items,spacing,[1,0],[0,0])
        
class TwoStateTextWidget(QWidget):
    ''' A text box that indicates state by color '''
    
    def __init__(self,parent=None,gstate=None,text='',readonly=True):
        ''' constructs the state box'''
        QWidget.__init__(self,parent)
        
        self._text = QLineEdit(text)
        self._text.setReadOnly(readonly)
        self._good = gstate
        self._state = False
        
    def update(self,state):
        ''' updates the state of the box '''
        if state == self._state: return
        if state == self._good:
            self._text.setStyleSheet("background-color: green")
            self._text.setText(str(state))
        else:
            self._text.setStyleSheet("background-color: red")
            self._text.setText(str(state))
        self._state = state == self._good


'''
Created on Dec 4, 2019

@author: Reynolds
'''

import os, sys

from configparser import ConfigParser

class wrt_settings(object):
    '''
    classdocs
    '''

    def __init__(self):
        '''
        Constructor
        '''
        self._error = False
        self._settings = {}
        self._parser = ConfigParser()
        
    def read(self,file):
        try:
            print('Reading %s ... ' % file,end='')
            if not os.path.exists(file): raise Exception('WRT Configuration file does not exist')
            self._parser.read(file)
            print('Done')
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno))
            self.write(file)# write the defaults
            
    
    def _parse(self):
        self.set('system_sensor',self._get_dict_value('SYSTEM','SENSOR',0,1704))
        self.set('system_ids',self._get_dict_value('SYSTEM','ids',0,['0'])) 
        self.set('system_id',int(self.get('system_ids')[0]))
        
        self.set('software_mode',self._get_dict_value('SETTINGS','mode',0,'operation'))
        
        self.set('sen_freq',self._get_dict_value('TOTALFIELD','freq',0,100))
        self.set('sen_time',self._get_dict_value('TOTALFIELD','time',0,10.0))
        self.set('sen_demean',self._get_dict_value('TOTALFIELD','demean',0,False))
        self.set('sen_filt',self._get_dict_value('TOTALFIELD','filter',0,'none'))
        self.set('sen_filt_win',self._get_dict_value('TOTALFIELD','window',0,15))
        self.set('sen_cutoff',self._get_dict_value('TOTALFIELD','filter',0,'none'))
        
    def set(self,key,value):
        try:
            self._settings[key] = value
        except Exception as e:
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno))
            
    def get(self,key=''):
        try:
            return self._settings[key]
        except Exception as e:
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno))
            return None
            
    def write(self,file):
        print(file)
    
    def errored(self):
        error = self._error
        self._error = False
        return error
    
    def _get_dict_value(self,section,value,index,default=-1):
        ''' returns a value from a dictionary '''
        try:
            if type(default) == type(float(0)):
                return float(self._parser[section][value][index].strip())
            elif type(default) == type(int(0)):
                return int(self._parser[section][value][index].strip())
            elif type(default) == type(''):
                return self._parser[section][value][index].strip()
            elif type(default) == type(True):
                return self._parser[section][value][index].strip() in ['true','True']
            elif type(default) == type([]):
                return self._parser[section][value]
            else: return default
        except Exception as e:
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno))
            return default
        
    def _cslist(self,l):
        ''' creates a comma separated list '''
        string = ''
        for ll in l:
            if type(ll) == type(' '): string += ll + ','
            else: string += str(ll) + ','
        return string[:-1]


'''
Created on Dec 5, 2019

@author: Reynolds
'''
import unittest
from wrtdk.parser.imu.microstrain import lord3dm_cv5


class Test(unittest.TestCase):


    def setUp(self):
        self.imu = lord3dm_cv5()


    def test_hasErrored(self):
        msg = b'ue\x80J\x0e\x0c\xc0\x1f\x0c0\xbfK\xcb\xaf\xbf\x80nq\x12\n=\xac="\xbfS>z>\x9e\xc31\xbe\xed\xe1\xf1\x0e\x04\xbf7\xa63>\xd9\xc8,?\r\x13{\x0e\x05;j\r\xbe\xbab\x9bG;~\x05\xbe\x0e\x06>\xe7\xbaI\xbe\xa3:4\xbd\x9b\xddibU'
        self.imu.parse(msg)
        
        self.assertFalse(self.imu.hasErrored())
        
    def test_get_parse(self):
        roll,pitch,yaw = 10,20,30
        
        m = self.imu.get(roll,pitch,yaw)
        
        self.imu.parse(m)
        
        self.assertFalse(self.imu.hasErrored())
        
        d = self.imu.getData()
        
        self.assertAlmostEqual(d[0],roll,1)
        self.assertAlmostEqual(d[1],pitch,1)
        self.assertAlmostEqual(d[2],yaw,1)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on Nov 19, 2019

@author: reynolds
'''
import os, sys, struct
from wrtdk.parser.parser import parser

class em(parser):
    
    PREAMBLE = b't'
    
    def __init__(self,count2mv=1):
        super().__init__()
        self._set = False
        self._count2mv = count2mv
        self._fmt = ''
        self.reset()
        
    def __str__(self):
        return '%s[tx:%d,rx:%d,ax:%d,fid:%d]'%(type(self).__name__,
                       self._tx,
                       self._rx,
                       self._ax,
                       self._ct)
        
    def reset(self):
        self._rx = self._minus1()
        self._tx = self._minus1()
        self._ax = self._minus1()
        self._ct = self._minus1()
        self._chksum = self._minus1()
        if not self.is_set(): self._tg = self._arr(0)
        
    def is_set(self):
        return self._set
        
    def set_bins(self,length=0):
        self._bins = length
        self._tg = self._arr(self._bins)
        self._set = True
    
    def get_em_index(self,b,rxs,axs):
        if not isinstance(b,tuple): tx,rx,ax = self.get_id_bits(b)
        elif len(b) is not 3: raise Exception('EMIndexException','The input tuple is not the proper length')
        else: tx,rx,ax = b
        #print('tx: %d rx: %d ax: %d %s'%(tx,rx,ax,self))
        return 3+tx*rxs*axs+ax*rxs+rx
    
    def get_tx(self):
        return self._tx
    
    def get_rx(self):
        return self._rx
    
    def get_axis(self):
        return self._ax
        
    def get_fiducial(self):
        return self._ct
    
    def get_imu_index(self):
        return 2
    
    def get_pos_index(self):
        return 1
    
    def get_conf_index(self):
        return 0
    
    def get_id_bits(self,b):
        return (0,0,0)
    
    def get_id_index(self):
        return 0
        
    def get_output_length(self,txs,rxs,axs):
        return 3+txs*rxs*axs
    
    def get_decay(self):
        return self._tg
    
    def get_decay_mv(self):
        return [i * self._count2mv for i in self._tg]
    
    def checksum(self):
        return 0
    
    def getData(self):
        ''' returns the data from the em message if set
        0) Tx
        1) Rx
        2) Ax
        3) Count
        4-end) millivolt bin data 
        '''
        if not self.is_set(): return []
        return [self._tx,self._rx,self._ax,self._ct]+self.get_decay_mv()

class emv1(em):
    '''
    classdocs
    '''
    
    TX_MASK = 0b11100000
    RX_MASK = 0b00011100
    AX_MASK = 0b00000011
    COUNT_TO_MV = .1526

    def __init__(self):
        '''
        Constructor
        '''
        super().__init__(count2mv=self.COUNT_TO_MV)
                
    def set_bins(self,length=0):
        super().set_bins(length=length)
        self._len = 1+1+1+self._bins*2+1
        self._fmt = '>cBB%dhx'%self._bins
    
    def get_id_bits(self, b):
        return ( ((b & self.TX_MASK) >> 5) - 1,
                 (b & self.RX_MASK) >> 2,
                 (b & self.AX_MASK) )
        
    def get_id_index(self):
        return 1
        
    def parse(self,msg):
        self.reset()
        
        try:
            if not self.is_set(): raise Exception('EMFormatException','Bin length is not set yet')
            elif msg[-1] != 10: raise Exception('EMFormatExcpetion',
                                            'Message is not terminated with new line feed')
            
            m = struct.unpack(self._fmt,msg)
            
            if m[0] != self.PREAMBLE: 
                raise Exception('EMFormatException','Message is not the proper format')
            
            self._tx,self._rx,self._ax = self.get_id_bits(m[1])
            self._ct = m[2]
            
            for i in range(self._bins): self._tg[i] = m[3+i]
                
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))    
    
class emv2(em):
    '''
    Identical to emv1 except with a checksum byte
    '''
    
    TX_MASK = 0b11100000
    RX_MASK = 0b00011100
    AX_MASK = 0b00000011
    COUNT_TO_MV = .1526

    def __init__(self):
        '''
        Constructor
        '''
        super().__init__(count2mv=self.COUNT_TO_MV)
                
    def get_id_bits(self, b):
        return ( ((b & self.TX_MASK) >> 5)-1,#this number is 1 indexed
                 (b & self.RX_MASK) >> 2,# zero indexed
                 (b & self.AX_MASK) )# zero indexed
        
    def get_id_index(self):
        return 2
    
    def checksum(self,byte_values):
        '''
        calculate a checksum value of a section of bytes
        '''
        total=0
        for b in byte_values:
            total+=b
        return total & 255
    
    def set_bins(self, length=0):
        super().set_bins(length=length)
        self._len = 1+1+1+1+self._bins*2+1
        self._fmt = '>cBBB%dhx'%self._bins
    
    def parse(self,msg):
        self.reset()
        
        try:
            if not self.is_set(): raise Exception('EMFormatException','Bin length is not set yet')
            elif msg[-1] != 10: raise Exception('EMFormatExcpetion',
                                            'Message is not terminated with new line feed')
            
            m = struct.unpack(self._fmt,msg)
            
            if m[0] != self.PREAMBLE: raise Exception('EMFormatException','Message is not the proper format')
            #checksum
            self._chksum = int(m[1])
            if self._chksum != self.checksum(msg[2:-1]):
                raise Exception('CONFVersionException',
                                'Checksum did not match')
            #cubestate
            self._tx,self._rx,self._ax = self.get_id_bits(m[2])
            #fiducial
            self._ct = m[3]
            
            for i in range(self._bins): self._tg[i] = m[4+i]
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))    


'''
Created on Apr 3, 2020

@author: Reynolds
'''
import unittest
from wrtdk.parser.em.em3d.conf.conf import confv5


class Test(unittest.TestCase):

    def setUp(self):
        self.msg = b'CONF\xa9\x05\x00\x00\x04\xea\x02\xfa\xe27\n\x05\x13\x88\x00\x1e\x11\x03 \x03\x06\x03\x10\x00\x05\x00\x10\x00!\x008\x00U\x00x\x00\xa1\x00\xd0\x01\x04\x01>\x01~\x01\xc4\x02\x10\x02b\x02\xba\x03 \n'

    def tearDown(self):
        pass

    def testParse(self):
        c = confv5()
        c.parse(self.msg)
        print(c.getData())
        self.assertFalse(c.hasErrored())

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on Dec 17, 2019

@author: Reynolds
'''

import os, time

class file_reader(object):
    '''
    classdocs
    '''
    
    def _elapsed_time(self,now):
        return time.time() - now
    
    def read(self,filename):
        now = time.time()
        if not os.path.exists(filename):
            print('%s does not exist. %fs'%(filename,self._elapsed_time(now)))
            return {}
        
        print('Reading %s ...' % filename)
        
        d = self._read(filename)
        
        if d: print('Done. %fs'%self._elapsed_time(now))
        else: print('Error: %fs'%self._elapsed_time(now))
            
        return d
        
    def _read(self,filename):
        pass


'''
Created on Dec 5, 2019

@author: Reynolds
'''
import unittest
from wrtdk.parser.msg.wrtmsg import udp_wrapper


class Test(unittest.TestCase):


    def setUp(self):
        self.wrapper = udp_wrapper()

    def test_hasErrored(self):
        msg = b'$SYSTM\x00\x00\xfbs\x00\x00\x13\x03L1[{\x1e\xcd\x00\x00\x00o\x00\x00\x00\x00\x00\x00\x00\x9c\x00\x00)\x1e'
        self.wrapper.parse(msg)
        
        self.assertFalse(self.wrapper.hasErrored())
        
    def test_get_parse(self):
        mtype = '$SYSTM'
        sen_id = 1001
        sys_id = 1234
        fid = 101
        status = 'AG'
        t_s = 10
        t_ms = 123
        mlen=42
        m = self.wrapper.get(mtype=mtype,sen_id=sen_id,sys_id=sys_id,
                             fid=fid,status=status,t_s=t_s, t_ms=t_ms,
                             mlen=mlen)
        
        self.wrapper.parse(m)
        
        self.assertFalse(self.wrapper.hasErrored())
        
        self.assertEqual(mtype,self.wrapper.getType())
        self.assertEqual(sen_id,self.wrapper.getSensorID())
        self.assertEqual(sys_id,self.wrapper.getSystemID())
        self.assertEqual(fid,self.wrapper.getFiducial())
        self.assertEqual(status,self.wrapper.getStatus())
        self.assertEqual(t_s+t_ms*1e-3,self.wrapper.getMCU())
        self.assertEqual(mlen,self.wrapper.getLength())

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on Jan 22, 2019

@author: reynolds
'''

import numpy as np
import matplotlib.pyplot as plt
from scipy.signal import medfilt, butter, filtfilt

class dsp_filter(object):
    ''' a super class for filtering data '''

    def __init__(self,ftype):
        ''' Sets the filter type '''
        self._type = ftype
        
    def filter(self,x):
        ''' filters the data '''
        return x
    
    def __str__(self):
        ''' returns the name of the filter '''
        return type(self).__name__
        
class no_filter(dsp_filter):
    ''' Performs no filtering to the data  '''
    
    def __init__(self):
        ''' sets the filter type to none '''
        super().__init__('NoFiilter')
        
    def filter(self,x):
        ''' returns the input without filtering it '''
        return x
    
class cutoff_filter(dsp_filter):
    ''' Performs an absolute cutoff filter by limiting bounds '''
    
    def __init__(self,cutoff):
        ''' sets the type to cutoff and sets the cutoff values '''
        super().__init__('CutoffFilter')
        self.setCutoff(cutoff)
        
    def setCutoff(self,cutoff):
        ''' sets the cutoff values '''
        self._cutoff = cutoff
        
    def filter(self,x):
        ''' returns the number if its inbetween the cutoff 
        values otherwise it returns an NaN '''
        y = x.copy()
        
        # cutoff filter the data
        for i,xx in np.ndenumerate(y): y[i] = max(min(xx,self._cutoff[1]),self._cutoff[0])
        
        return y
        
class median_fitler(dsp_filter):
    ''' Performs a median filter '''
    
    def __init__(self,window=15):
        ''' sets the filter type and window size '''
        super().__init__('MedianFilter')
        self.setWindow(window)
        
    def setWindow(self,window=15):
        ''' sets the window size '''
        self._win = window
    
    def filter(self,x):
        ''' run a median filter over the data '''
        return medfilt(x,self._win)

class butter_filter(dsp_filter):
    
    def __init__(self,lowcut,highcut,fs,order=4, *args, **kwargs):
        super().__init__('butterworth_filter')
        nyq = 0.5 * fs
        low = lowcut / nyq
        high = highcut / nyq
        
        if 'bandpass' in kwargs.keys() or 'bandstop' in kwargs.keys():
            cutoff = [low,high]
        else: cutoff = [low]
        
        self.b,self.a = butter(order,cutoff,*args,**kwargs)
        
    def filter(self,x):
        return filtfilt(self.b,self.a,x)

if __name__.startswith('__main__'):
    t = np.arange(100)
    x = 100.0*np.random.rand(len(t))+52e3
    
    nfilt = no_filter()
    x1 = nfilt.filter(x)
    
    mfilt = median_fitler(15)
    x2 = mfilt.filter(x)
    
    cfilt = cutoff_filter([52050,52080])
    x3 = cfilt.filter(x-np.mean(x))
    
    t1 = np.arange(0,.4,0.01)
    y1 = 10*np.cos(2*np.pi*50*t1)
    y2 = 8 * np.cos(2*np.pi*4*t1)
    y3 = y1 + y2
    
    bfilt = butter_filter(40,50,100,8,btype='lowpass',analog=False)
    x4 = bfilt.filter(y3)
    
    #plt.plot(t,x,t,x1,t,x2,t,x3)#,t1,y1)
    #plt.legend(['original','none','median','cutoff'])
    #plt.show()
    
    plt.plot(t1,y3,t1,x4)
    plt.show()
    


'''
Created on Aug 17, 2018

@author: reynolds
'''

import os, sys, struct
from wrtdk.parser.parser import parser
from wrtdk.parser.msg.wrtmsg import udp_wrapper
from wrtdk.io.sim.sim_simulator import sim_simulator

class mx3g(parser):
    ''' parser for the mfam mx3g message type'''

    def __init__(self):
        ''' Constructor '''
        super().__init__()# inherit superclass
        
        # initialize properites
        self.reset()
        
        # define constants
        self.COUNT_2_NT_VMAG = 1#1000 / 75
        self.NT_VMAG_2_COUNT = 1.0/self.COUNT_2_NT_VMAG
        self.COUNT_2_DEG_C = 1
        self.DEG_C_2_COUNT = 1.0/self.COUNT_2_DEG_C
        self.COUNT_2_NT = 50e-6
        self.NT_2_COUNT = 20000.0
        
    def reset(self):
        ''' resets the parser '''
        self._pps = self._minus1()
        self._synch = self._minus1()
        self._failed = self._minus1()
        self._count = self._minus1()
        self._status1 = self._minus1()
        self._status2 = self._minus1()
        self._mag1 = self._nan()
        self._stat1 = self._minus1()
        self._v1 = self._minus1()
        self._mag2 = self._nan()
        self._stat2 = self._minus1()
        self._v2 = self._minus1()
        self._bx = self._nan()
        self._by = self._nan()
        self._bz = self._nan()
        self._temperature = self._nan()
        self._a0 = self._minus1()
        self._a1 = self._minus1()
        self._a2 = self._minus1()
        self._checksum = self._minus1()
    
    def get(self,m1=0,m2=0,v1=0,v2=0,s1=0,s2=0,count=0,failed=0,synch=0,pps=0,a0=0,a1=0,a2=0,adc0=0,adc1=0,adc2=0,temp=0):
        int1 = (count & 0x03ff) | ((v2&0x0001) << 15) | ((v1&0x0001) << 14) | ((a2&0x0001) << 13) | ((a1&0x0001) << 12) | ((a0&0x0001) << 11)
        pps = ((pps & 0x0001) << 15) | ((synch & 0x0001) << 14) | (failed & 0x0001)
        return struct.pack('>HHihhihhhhi',
                           int1,
                           pps,
                           int(m1*self.NT_2_COUNT),
                           s1,
                           s2,
                           int(m2*self.NT_2_COUNT),
                           int(adc0*self.NT_VMAG_2_COUNT),
                           int(adc1*self.NT_VMAG_2_COUNT),
                           int(adc2*self.NT_VMAG_2_COUNT),
                           int(temp*self.DEG_C_2_COUNT),
                           0)
        #fid,stat,self._mag1,stat1,stat2,self._mag2,self._bx,self._by,self._bz,self._temperature,_ = struct.unpack('>HHihhihhhhi',msg)
        #fid,stat,self._mag1,stat1,stat2,self._mag2,self._bx,self._by,self._bz,self._temperature,_ = struct.unpack('<HHihhihhhhi',msg)
    
    def parse(self,msg):
        ''' parses the messages from the mx3g sensor platform '''
        self.reset()
        try:
            fid,stat,self._mag1,stat1,stat2,self._mag2,self._bx,self._by,self._bz,self._temperature,_ = struct.unpack('>HHihhihhhhi',msg)
            self._v1 = (fid >> 14) & 0x0001
            self._v2 = (fid >> 15) & 0x0001
            self._a2 = (fid >> 13) & 0x0001
            self._a1 = (fid >> 12) & 0x0001
            self._a0 = (fid >> 11) & 0x0001
            self._count = fid & 0x03ff
            self._pps = stat >> 15
            self._synch = (stat >> 14) & 0x01
            self._failed = stat & 0x0001
            self._mag1  *= self.COUNT_2_NT
            self._status1 = stat1
            self._status2 = stat2
            self._mag2 *= self.COUNT_2_NT
            self._bx *= self.COUNT_2_NT_VMAG
            self._by *= self.COUNT_2_NT_VMAG
            self._bz *= self.COUNT_2_NT_VMAG
            self._temperature *= self.COUNT_2_DEG_C
            self._checksum = self._calculateChecksum(msg)
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
            
    def getData(self):
        ''' returns the data from the message 
        0) mag 1 total field in nT
        1) mag 2 total field in nT
        2) bx in nT
        3) by in nT
        4) bz in nT
        5) temperature in F
        6) mag1 status
        7) mag2 status
        8) Reserved For Future Use (-1)
        9) count
        10) pps flag
        11) synched flag
        12) failed flag
        13) Mag 1 valid
        14) Mag 2 valid 
        15) A0
        16) A1
        17) A2
        '''
        return [self._mag1,self._mag2,
                self._bx,self._by,self._bz,
                self._temperature,
                self._status1,self._status2,
                -1,self._count,self._pps,
                self._synch,self._failed,
                self._v1,self._v2,
                self._a0,self._a1,self._a2]
    
    def debug(self):
        ''' debugs the message '''
        return 'id:%4d n:%4d m1:%10.3f s1:%2d s2:%2d m2:%10.3f' % (self._id,
                                                                   self._count,
                                                                   self._mag1,
                                                                   self._status1,
                                                                   self._status2,
                                                                   self._mag2)
    
class mfamfile(mx3g):
    ''' parses the mfam messages from a file '''
    
    def __init__(self):
        ''' Constructor '''
        super().__init__()# inherit superclass
    
    def parse(self, msg):
        ''' little endian parser '''
        
        try:
            fid,stat,self._mag1,stat1,stat2,self._mag2,self._bx,self._by,self._bz,self._temperature,_ = struct.unpack('<HHihhihhhhi',msg)
            self._id = (0b11111000 & (fid)) >> 3
            self._count = (fid >> 8) & 0xff + ((fid & 0b00000111) << 8)
            self._pps = ((stat & 0xff) & 0b10000000) >> 7
            self._synch = ((stat & 0xff) & 0b01000000) >> 6
            self._failed = ((stat >> 8) & 0b00000001)
            self._mag1 *= self.COUNT_2_NT
            self._status1 = stat1#(stat1 & 0b11100000) >> 5
            self._status2 = stat2#(stat2 & 0b11100000) >> 5
            self._mag2 *= self.COUNT_2_NT
            self._bx *= self.COUNT_2_NT_VMAG
            self._by *= self.COUNT_2_NT_VMAG
            self._bz *= self.COUNT_2_NT_VMAG
            self._temperature *= self.COUNT_2_DEG_C
            self._checksum = self._calculateChecksum(msg)
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))

def test_mfamfile():
    print('testing mfamfile ...')
    tf = mfamfile()
    filename = r'C:\Users\reynolds\Documents\1704_madunit\data\20180830_mfam\MFAM24'
    with open(filename,'rb') as f:
        n = 4
        spcr = 4
        off = 16
        mlen = 28
        msg = f.read(off+n*mlen+spcr*n)
        
        for i in range(n):
            start = off+(i)*mlen + spcr*i
            end = off+(i+1)*mlen + spcr*i
            m = msg[start:end]
            tf.parse(m)
            if not tf.hasErrored():
                print('%4d:%4d %s %3d %s' % 
                      (start,
                       end,
                       tf.debug(),
                       len(msg[start:end]),
                       str(msg[start:end])))
            else:
                print('%4d:%4d %s' % (start,end,str(msg[start:end])) )
                
def test_m3xg():
    m = mx3g()
    f = r'C:\Users\reynolds\Documents\1704_madunit\data\20190327_mfam-test\madunit_mfam_test_1.dat'
    print(os.path.exists(f))
    msg = get_msg(f,20)
    for mm in msg:
        print('len: %d msg:%s' % (len(mm),mm))
        m.parse(mm)
        print(m.getData())
    
def get_msg(filename,n=1):
    reader = sim_simulator()
    reader.read(filename,'$MFAM1')
    payload = []
    wrapper = udp_wrapper()
    while len(payload) < n:
        msg,dt = reader.getNext()
        wrapper.parse(msg[0:wrapper.LENGTH])
        if not wrapper.hasErrored():
            payload.append(msg[wrapper.getMsgStart():wrapper.getMsgEnd()])
    
    return payload
                
if __name__ == '__main__':
    test_mfamfile()
    print()
    test_m3xg()


import os

def print_error(exc_type, unknown, exc_tb,err='',msg=''):
    fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
    print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(err), fname, exc_tb.tb_lineno,msg))   


'''
Created on Aug 17, 2018

@author: reynolds
'''

import os, sys, struct
from wrtdk.parser.parser import parser
from wrtdk.parser.msg.wrtmsg import udp_wrapper

class bno055(parser):
    ''' a parser for the wrt bno055 message '''

    def __init__(self):
        ''' Constructor '''
        super().__init__()# inherit initial class
        
        # define constants
        self.COUNT_2_DEGREES = 1/16
        self.COUNT_2_CELCIUS = 1
        
        #reset
        self.reset()
        
    def reset(self):
        ''' resets the parser '''
        self._y = self._nan()
        self._p = self._nan()
        self._r = self._nan()
        self._t = self._nan()
        self._sys = self._minus1()
        self._acc = self._minus1()
        self._gyr = self._minus1()
        self._mag = self._minus1()
        
    def parse(self,msg):
        ''' parses the messages '''
        self.reset()
        
        try:
            y,r,p = struct.unpack('>hhh',msg[18:24])#msg[54:60])
            t,cal = struct.unpack('>bB',msg[44:46])#msg[80:82])
            self._r = r * self.COUNT_2_DEGREES
            self._p = p * self.COUNT_2_DEGREES
            self._y = y * self.COUNT_2_DEGREES
            self._t = t * self.COUNT_2_CELCIUS
            self._sys = (cal&0b11000000)>>6
            self._acc = (cal&0b00110000)>>4
            self._gyr = (cal&0b00001100)>>2
            self._mag = (cal&0b00000011)
            
            self._checksum = self._calculateChecksum(msg)
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
            
    def getData(self):
        ''' returns the data 
        0) roll in degrees
        1) pitch in degrees
        2) yaw in degrees
        3) temperature in celcius
        4) system calibration value
        5) accelerometer calibration value
        6) gyro calibration value
        7) mag calibration value'''
        return [self._r,self._p,self._y,
                self._t,
                self._sys,self._acc,self._gyr,self._mag]
    
def test_bno055():
    imu = bno055()
    filename = r'C:\Users\reynolds\Documents\1711_nuwc\data\20180732_deckcheck\20180731_deckcheckS01.dat'
    msg = get_msg(filename)
    print('msg:%s' % msg)
    imu.parse(msg)
    print(imu.getData())
    
def get_msg(filename):
    with open(filename,'rb') as f:
        idlen = 6
        msg = f.read(idlen)
        pos = 6
        
        while msg != b'$IMUXX' and pos < 1000:
            msg = msg[1::] + f.read(1)
            pos = pos + 1
            
        wrapper = udp_wrapper()
            
        msg = msg + f.read(36-6)
        pos = pos + 36 - 6
        wrapper.parse(msg)
        
        payload = f.read(wrapper.getLength())
        pos = pos + 1
        return payload
            
if __name__ == '__main__':
    test_bno055()


'''
Created on Oct 16, 2018

@author: reynolds
'''
import time, os
import numpy as np

from wrtdk.instrument.instrument import instrument

class N9918A(instrument):
    '''
    classdocs
    '''
    _ifft = 4096

    def __init__(self, params):
        ''' Constructor '''
        super().__init__(params)
        
        self._open = False
        self.verb = self._params['verb']
        self.dstr = ''
        self.trace = 0
        self.data = np.array([])
        
        print('Attempting to connect to the Field Fox at',self._params['addr'],'...',end=' ')
        self._device = self._rm.open_resource('TCPIP0::%s::inst0::INSTR' % self._params['addr'])
        self._wait()
        
        self._device.chunk_size = self._params['chunk_size']
        self._device.timeout = self._params['timeout']
        self._device.write_termination = '\r\n'
        self._device.read_termination = '\n'
        self._device.write('*CLS')
        if self.verb: print('')# add a space
        if self.verb: print('*IDN:',self._device.query("*IDN?").strip())
        if self.verb: print('chunk_size: %d' % self._device.chunk_size)
        if self.verb: print('timeout: %d ms' % self._device.timeout)
        if self.verb: print('read_termination: %s' % repr(self._device.read_termination))
        if self.verb: print('write_termination: %s' % repr(self._device.write_termination))
        if self.verb: print('send_end: %r' % self._device.send_end)
        if self.verb: print('query_delay: %d s' % self._device.query_delay)
        print('Done\n')
        
        self._hdrlist = []
        self._hdr = ''
        
        print('Configuring the Field Fox ... ')
        while True:
            try:
                # begin configuration
                self._device.write('INST:SEL "NA"')# set the network analyzer mode
                self._wait()#time.sleep(5)# sleep to allow the program to load
                self._hdrlist.append('SEL: %s' %self._device.query('INST:SEL?').strip())
                self._wait()
                
                # set the sweep settings
                self._device.write('FREQ:STAR %f' % self._params['fstart'])
                self._device.write('FREQ:STOP %f' % self._params['fstop'])
                self._device.write('BWID %f' % self._params['bwid'])
                self._device.write('FREQ:CENT %f' % self._params['fcen'])
                self._device.write('FREQ:SPAN %f' % self._params['fspn'])
                self._device.write('SWE:POIN %d' % self._params['np'])
                self._device.write('SOUR:POW:ALC MAN')
                self._device.write('SOUR:POW %.1f' % self._params['spwr'])
                self._device.write('CALC:TRAN:TIME:STAT %s' % self._params['trans'])
                
                # get the transform information
                if self._params['trans'] is 'OFF':# frequency domain
                    self.x_units = 'Hz'
                    self.start = float(self._device.query('FREQ:STAR?'))
                    self.stop = float(self._device.query('FREQ:STOP?'))
                else:# time domain
                    self.start = float(self._device.query('CALCulate:TRANsform:TIME:STARt?')) # get f start and stop
                    self.stop = float(self._device.query('CALC:TRANsform:TIME:STOP?'))
                    self.x_units = 's'
                
                # print the x axis settings to the user
                self._hdrlist.append('BWID: %s Hz' % self._device.query('BWID?').strip())
                self._hdrlist.append('FREQ:STAR: %s Hz' % self._device.query('FREQ:STAR?').strip())
                self._hdrlist.append('FREQ:STOP %s Hz' % self._device.query('FREQ:STOP?').strip())
                self._hdrlist.append('FREQ:SPAN %s Hz' % self._device.query('FREQ:SPAN?').strip())
                self._hdrlist.append('FREQ:CENT %s Hz' % self._device.query('FREQ:CENT?').strip())
                
                self._hdrlist.append('SWE:TIME %s s' % self._device.query('SWE:TIME?').strip())
                self._hdrlist.append('FREQ:MTIM %s s' % self._device.query('SWE:MTIM?').strip())
                self._hdrlist.append('SOUR:POW %s dBm' % self._device.query('SOUR:POW?').strip())
                self._hdrlist.append('SOUR:POW:ALC %s' % self._device.query('SOUR:POW:ALC?').strip())
                self._hdrlist.append('TRANSFORM: % s' % self._device.query('CALC:TRAN:TIME:STAT?').strip())
                self._hdrlist.append('AVER:MODE %s' % self._device.query('AVER:MODE?').strip())
                self._hdrlist.append('AVER:COUN %s' % self._device.query('AVER:COUN?').strip())
                self._hdrlist.append('CALC:SMO %s' % self._device.query('CALC:SMO?').strip())
                self._hdrlist.append('CALC:SMO:APER %s' % self._device.query('CALC:SMO:APER?').strip())
                self._hdrlist.append('SWE:POIN %s' % self._device.query('SWE:POIN?').strip())
                self._hdrlist.append('CALC:TRAN:TIME:STAR %s s' % self._device.query('CALCulate:TRANsform:TIME:STARt?'))
                self._hdrlist.append('CALC:TRAN:TIME:STOP %s s' % self._device.query('CALC:TRANsform:TIME:STOP?'))
                self.n = float(self._device.query('SENS:SWE:POIN?'))# tell the user the number of points
                self.axis = np.linspace(self.start,self.stop,self.n)# set the x vector
                self.dx = np.mean(np.diff(self.axis))
                self.time = np.linspace(0,1.0/self.dx,self._ifft)
                self.xdata = np.concatenate((np.array([np.NaN,np.NaN]),self.axis)).reshape((int(self.n)+2,1))
                self.window = np.hanning(200)
                
                # setup the display
                self._device.write('CALC:FORM %s' % self._params['form'])
                self._hdrlist.append('CALC:FORM: %s' % self._device.query('CALC:FORM?').strip())
                self._device.write('DISP:WIND:TRACe1:Y:AUTO')# set the plot to autoscale
                
                # setup the transmission type
                self._device.write('CALC:PAR1:DEF S12')# set the measurement parameter
                self._hdrlist.append('TX: %s' % self._device.query('CALC:PAR:DEF?').strip())
                self._device.write('CALC:PAR1:SEL')# set the active trace
                self._device.write('FORM %s' % self._params['dform'])# REAL,64 for binary
                self._hdrlist.append('FORM: %s' % self._device.query('FORM?').strip())
                
                # set trigger
                self._hdrlist.append('CONT: %s' % self._device.query('INIT:CONT?').strip())
                self._device.write('INIT:CONT 0')# set non-continuous
                self._hdrlist.append('AVER: %s' % self._device.query('AVER:COUNt?').strip())
                
                # create the header and print if verbose
                for i in self._hdrlist: self._hdr += i + '\n'
                if self.verb: print(self._hdr,end='')
                
                # wait for everything and clear the buffer
                self._wait()
                self._clear()
                self._open = True
            except Exception as e:
                print('Error.',str(e))
                # reopen the device
                self.reopen_inst()
                #time.sleep(0.5)
            else:
                break
            
        self._open = True
        print('Setup complete.\n')
        
    def isOpen(self):
        ''' returns whether the port is open '''
        return self._open
        
    def measure(self):
        ''' measures the current data '''
        instrument.measure(self)
        
        try:
            self._clear()# clear the buffer
                            # get the data from the instrument
            if self.verb: print('Starting measurement ... ',end='')
            self._device.write('INIT:IMMediate')# trigger a measurement
            self._wait()
                
            if self.verb: print('ascan%d ... ' % self.trace,end=' ')
            
            #self._device.write('MMEMory:STORe:FDATa "ascan.csv"')
            if self._params['dform'] == "REAL,64":
                self._device.write('CALC:DATA:SDATa?')
                self.dstr = self._device.read_bytes(int(self.n)*2*8+7)
                data = np.frombuffer(self.dstr[6:-1]).reshape((int(self.n),2))
            elif self._params['dform'] == 'REAL,32':
                self._device.write('CALC:DATA:SDATa?')
                self.dstr = self._device.read_bytes(int(self.n)*2*4+7)
                data = np.frombuffer(self.dstr[6:-1],dtype=np.float32).reshape((int(self.n),2))
            else:
                self.dstr = self._device.query('CALC:DATA:SDATa?')
                data = np.fromstring(self.dstr,sep=',').reshape((int(self.n),2))
                
            self.data = np.vstack((np.array([np.NaN,np.NaN,np.NaN,np.NaN]).reshape(2,2),data))
            #self.data = np.column_stack((self.data,dout))
            if self.verb: print('Done')
                
            self.trace = self.trace + 1
                
            return True
        except Exception as e:
            print('Error.',str(e))
            #self.reopen_inst()
            #time.sleep(0.5)
            return False
        
    def getData(self):
        ''' returns the data 
        0) x data
        1) trace data 
        2) number of samples
        3) time in seconds
        4) ascan
        '''
        # process the ascan
        a = self.data[2:,0] + 1j*self.data[2:,1]
        ww = np.ones((int(self.n)))
        ww[0:100] = self.window[0:100]
        ascan = np.fft.ifft(ww*a,self._ifft)# taken from MATLAB
        return [self.axis,self.data[2:,:],int(self.n),self.time,ascan]
        
    def write(self, fn='',odir='',d=np.nan,t=np.nan):
        ''' writes a measurement to a file '''
        instrument.write(self, fn=fn)
        
        # insert filename if it does not exist
        if fn == '': fn = r'bscan.csv'
        
        # set the directory path
        odir = os.path.join(os.getcwd(),odir)
        if not os.path.exists(odir): os.mkdir(odir)
        fn = os.path.join(odir,fn)
        
        # write the data
        print('Writing %s ... ' % fn,end='')
        try:
            self.data[0][0] = t
            self.data[0][1] = t
            self.data[1][0] = d
            self.data[1][1] = d
            
            # append to the file
            f = open(fn,'a')
            np.savetxt(f,self.data.T,delimiter=',')
            os.chmod(fn,0o666)# don't know what this does
            f.close()
            print('Done')
        except Exception as e:
            print('Error',str(e))
            
    def header(self, fn='',odir=''):
        ''' writes the header '''
        
        # set the directory path
        odir = os.path.join(os.getcwd(),odir)
        if not os.path.exists(odir): os.mkdir(odir)
        fn = os.path.join(odir,fn)
        
        # write header to file
        try:
            print('Writing %s header ... ' % fn,end='')
            f = open(fn,'a')
            f.write(self._hdr)
            np.savetxt(f,self.xdata.T,delimiter=',')
            os.chmod(fn,0o666)# don't know what this does
            f.close()
            self.resetTrace()
            print('Done')
        except Exception as e:
            print('Error',str(e))
            
    def resetTrace(self):
        self.trace = 0
            
    def _clear(self):
        ''' clears the buffer '''
        try:
            self._device.write('*CLS')
        except:
            return
            
    def _wait(self):
        ''' waits for the fieldfox to finish '''
        try:
            return self._device.query('*OPC?').strip()
        except:
            return 'Error'
            
    def reopen_inst(self):
        ''' reopens the instance of the instrument '''
        while True:
            try:
                self._device = self._rm.open_resource('TCPIP0::%s::inst0::INSTR' % self._params['addr'])
            except:
                print('resource busy?')
                print('DEBUG:','')
                self._device.close()
                time.sleep(1)
            else:
                self._open = False
                break
        
    def oppar(self,par):
        ''' checks the parameters in '''
        instrument.oppar(self,par)
        if ('timeout' not in par): par['timeout'] = None
        if ('verb' not in par): par['verb'] = False
        if ('py' not in par): par['py'] = True
        if ('np' not in par): par['np'] = 801
        if ('addr' not in par): par['addr'] = '169.254.10.100'
        if ('trans' not in par): par['trans'] = 'ON'
        if ('form' not in par): par['form'] = 'REAL'
        if ('chunk_size' not in par): par['chunk_size'] = 102400
        if ('fstart' not in par): par['fstart'] = 30e3
        if ('fstop' not in par): par['fstop'] = 12e9
        if ('bwid' not in par): par['bwid'] = 10e3
        if ('fcen' not in par): par['fcen'] = 6000015000
        if ('fspn' not in par): par['fspn'] = 11999970000
        if ('spwr' not in par): par['spwr'] = -2
        
    def close(self):
        ''' closes the device '''
        print('Closing connection to %s ... ' % self._params['addr'],end='')
        self._device.close()
        self._rm.close()
        self._open = False
        print('Done')
        
if __name__ == '__main__':
    print('Starting fieldfox comms')
    par = dict(py=True,
               verb=True,
               addr='128.128.204.202',
               trans='OFF',
               form='POL',
               timeout=2000,
               chunk_size=102400,
               dform='REAL,64',
               np=401,
               spwr=-2)
    fn = 'test6.csv'
    odir='20180102_gpr'
    ff = N9918A(par)
    ff.header(fn=fn,odir=odir)
    for _ in range(1):#190 will do down and back on the gantry
        if ff.measure():
            ff.write(fn=fn,odir=odir,d=np.NaN,t=time.time())
    
    ff.close()
    print('Shutting down')


'''
Created on Apr 8, 2020

@author: Reynolds
'''

import threading, time, os

class simulator(threading.Thread):
    '''
    classdocs
    '''

    def __init__(self,port=[]):
        '''
        Constructor
        '''
        super().__init__()
        self._port = port
        self._msgs = []
        self._dt = []
        self._current = 0
        self._running = False
        self._error = False
        
    def _append(self,msg,dt):
        self._msgs.append(msg)
        self._dt.append(dt)
        
    def read(self,filename):
        print('Reading %s ... ' % filename)
        if not os.path.exists(filename): 
            self._error = True
            print('Error. %s does not exist' % filename)
        
    def run(self):
        threading.Thread.run(self)
        
        if self._error: return
        
        if not self._port: 
            print('No port was entered')
            return
        
        self._running = True
        while self._running:
            try:
                self._port.write(self._msgs[self._current])
                time.sleep(self._dt[self._current])
                self._current += 1
                if self._current >= len(self._msgs): self._current = 0
            except Exception as e:
                print(str(e))
                time.sleep(1)
                continue